from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import get_crowdstrike_field
from pypanther.log_types import PantherLogType

crowdstrike_real_time_response_session_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="RTS session start event",
        ExpectedResult=True,
        Log={
            "cid": "12345abcdef",
            "unknown_payload": {
                "AgentIdString": "12ab56cd",
                "CustomerIdString": "1234",
                "EventType": "Event_ExternalApiEvent",
                "ExternalApiType": "Event_RemoteResponseSessionStartEvent",
                "HostnameField": "John Macbook Pro",
                "Nonce": -4714046577736361000,
                "SessionId": "6e1181e4-4924-4761-az3d-666851jdb950",
                "StartTimestamp": 1670460538,
                "UTCTimestamp": 1670460538000,
                "UserName": "example@example.io",
                "cid": "12345abcdef",
                "eid": 118,
                "timestamp": "2022-12-08T00:48:58Z",
            },
        },
    ),
    PantherRuleTest(
        Name="RTS session not started",
        ExpectedResult=False,
        Log={
            "cid": "12345abcdef",
            "unknown_payload": {
                "AgentIdString": "12ab56cd",
                "CustomerIdString": "1234",
                "EventType": "Event_ExternalApiEvent",
                "ExternalApiType": "Event_RemoteResponseSessionEndEvent",
                "HostnameField": "John Macbook Pro",
                "Nonce": -4714046577736361000,
                "SessionId": "6e1181e4-4924-4761-az3d-666851jdb950",
                "StartTimestamp": 1670460538,
                "UTCTimestamp": 1670460538000,
                "UserName": "example@example.io",
                "cid": "12345abcdef",
                "eid": 118,
                "timestamp": "2022-12-08T00:48:58Z",
            },
        },
    ),
    PantherRuleTest(
        Name="RTS session start event (FDREvent)",
        ExpectedResult=True,
        Log={
            "event": {
                "AgentIdString": "42db160eec7948658374a28a4088f297",
                "CustomerIdString": "712bcd164963442ea43d52917cecdecc",
                "EventType": "Event_ExternalApiEvent",
                "ExternalApiType": "Event_RemoteResponseSessionStartEvent",
                "HostnameField": "US-C02TEST",
                "Nonce": "13732697495973190000",
                "SessionId": "6e1081e4-4914-4761-af3d-666851adb950",
                "StartTimestamp": "1670460538",
                "UTCTimestamp": "1670460538000",
                "UserName": "someone@runpanther.io",
                "cid": "",
                "eid": "118",
                "timestamp": "2022-12-08T00:48:58Z",
            },
            "timestamp": "2022-12-08 00:48:58.000000000",
            "aid": "42db160eec7948658374a28a4088f297",
            "cid": "712bcd164963442ea43d52917cecdecc",
            "fdr_event_type": "Event_RemoteResponseSessionStartEvent",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_event_time": "2022-12-08T00:48:58Z",
            "p_any_domain_names": ["US-C02TEST"],
            "p_any_md5_hashes": [
                "42db160eec7948658374a28a4088f297",
                "712bcd164963442ea43d52917cecdecc",
            ],
            "p_any_trace_ids": [
                "42db160eec7948658374a28a4088f297",
                "712bcd164963442ea43d52917cecdecc",
            ],
            "p_any_usernames": ["someone@runpanther.io"],
            "p_any_emails": ["someone@runpanther.io"],
        },
    ),
    PantherRuleTest(
        Name="RTS session not started (FDREvent)",
        ExpectedResult=False,
        Log={
            "event": {
                "AgentIdString": "42db160eec7948658374a28a4088f297",
                "CustomerIdString": "712bcd164963442ea43d52917cecdecc",
                "EventType": "Event_ExternalApiEvent",
                "ExternalApiType": "Event_RemoteResponseSessionEndEvent",
                "HostnameField": "US-C02TEST",
                "Nonce": "13732697495973190000",
                "SessionId": "6e1081e4-4914-4761-af3d-666851adb950",
                "StartTimestamp": "1670460538",
                "UTCTimestamp": "1670460538000",
                "UserName": "someone@runpanther.io",
                "cid": "",
                "eid": "118",
                "timestamp": "2022-12-08T00:48:58Z",
            },
            "timestamp": "2022-12-08 00:48:58.000000000",
            "aid": "42db160eec7948658374a28a4088f297",
            "cid": "712bcd164963442ea43d52917cecdecc",
            "fdr_event_type": "Event_RemoteResponseSessionEndEvent",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_event_time": "2022-12-08T00:48:58Z",
            "p_any_domain_names": ["US-C02TEST"],
            "p_any_md5_hashes": [
                "42db160eec7948658374a28a4088f297",
                "712bcd164963442ea43d52917cecdecc",
            ],
            "p_any_trace_ids": [
                "42db160eec7948658374a28a4088f297",
                "712bcd164963442ea43d52917cecdecc",
            ],
            "p_any_usernames": ["someone@runpanther.io"],
            "p_any_emails": ["someone@runpanther.io"],
        },
    ),
]


class CrowdstrikeRealTimeResponseSession(PantherRule):
    DisplayName = "Crowdstrike Real Time Response (RTS) Session"
    RuleID = "Crowdstrike.RealTimeResponse.Session-prototype"
    Severity = PantherSeverity.Medium
    LogTypes = [PantherLogType.Crowdstrike_Unknown, PantherLogType.Crowdstrike_FDREvent]
    Tags = ["Crowdstrike"]
    Description = "Alert when someone uses Crowdstrike’s RTR (real-time response) capability to access a machine remotely to run commands.\n"
    Runbook = "Validate the real-time response session started by the Actor.\n"
    Reference = "https://falcon.us-2.crowdstrike.com/documentation/71/real-time-response-and-network-containment#reviewing-real-time-response-audit-logs"
    Tests = crowdstrike_real_time_response_session_tests

    def rule(self, event):
        return (
            get_crowdstrike_field(event, "ExternalApiType", default="<unknown-ExternalApiType>")
            == "Event_RemoteResponseSessionStartEvent"
        )

    def title(self, event):
        user_name = get_crowdstrike_field(event, "UserName", default="<unknown-UserName>")
        hostname_field = get_crowdstrike_field(
            event, "HostnameField", default="<unknown-HostNameField>"
        )
        return (
            f"{user_name} started a Crowdstrike Real-Time Response (RTR) shell on {hostname_field}"
        )

    def alert_context(self, event):
        return {
            "Start Time": get_crowdstrike_field(
                event, "StartTimestamp", default="<unknown-StartTimestamp>"
            ),
            "SessionId": get_crowdstrike_field(event, "SessionId", default="<unknown-SessionId>"),
            "Actor": get_crowdstrike_field(event, "UserName", default="<unknown-UserName>"),
            "Target Host": get_crowdstrike_field(
                event, "HostnameField", default="<unknown-HostnameField>"
            ),
        }



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

duo_admin_bypass_code_created_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Bypass Create",
        ExpectedResult=True,
        Log={
            "action": "bypass_create",
            "description": '{"bypass": "", "count": 1, "valid_secs": 3600, "auto_generated": true, "remaining_uses": 1, "user_id": "D12345", "bypass_code_ids": ["A12345"]}',
            "isotimestamp": "2022-12-14 21:17:39",
            "object": "target@example.io",
            "timestamp": "2022-12-14 21:17:39",
            "username": "Homer Simpson",
        },
    ),
    PantherRuleTest(
        Name="Bypass Delete",
        ExpectedResult=False,
        Log={
            "action": "bypass_detele",
            "description": '{"bypass": "", "count": 1, "valid_secs": 3600, "auto_generated": true, "remaining_uses": 1, "user_id": "D12345", "bypass_code_ids": ["A12345"]}',
            "isotimestamp": "2022-12-14 21:17:39",
            "object": "target@example.io",
            "timestamp": "2022-12-14 21:17:39",
            "username": "Homer Simpson",
        },
    ),
]


class DuoAdminBypassCodeCreated(PantherRule):
    Description = "A Duo administrator created an MFA bypass code for an application."
    DisplayName = "Duo Admin Bypass Code Created"
    Runbook = "Confirm this was authorized and necessary behavior."
    Reference = "https://duo.com/docs/administration-users#generating-a-bypass-code"
    Severity = PantherSeverity.Medium
    LogTypes = [PantherLogType.Duo_Administrator]
    RuleID = "Duo.Admin.Bypass.Code.Created-prototype"
    Tests = duo_admin_bypass_code_created_tests

    def rule(self, event):
        # Return True to match the log event and trigger an alert.
        return event.get("action", "") == "bypass_create"

    def title(self, event):
        # If no 'dedup' function is defined, the return value of
        # this method will act as deduplication string.
        return f"Duo: [{event.get('username', '<NO_USER_FOUND>')}] created a MFA bypass code for [{event.get('object', '<NO_OBJECT_FOUND>')}]"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

carbon_black_audit_flagged_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Flagged",
        ExpectedResult=True,
        Log={
            "clientIp": "12.34.56.78",
            "description": "User bob.ross@acme.com retrieved secret for API ID JFDNIPS464 in org 12345",
            "eventId": "66443924833011eeac3cb393f3d07f9f",
            "eventTime": "2023-11-14 20:57:19.186000000",
            "flagged": True,
            "loginName": "bob.ross@acme.com",
            "orgName": "acme.com",
            "verbose": False,
        },
    ),
    PantherRuleTest(
        Name="Other role granted",
        ExpectedResult=False,
        Log={
            "clientIp": "12.34.56.78",
            "description": "Created grant: psc:cnn:A1234567:BC1234567890 with role Read Only",
            "eventId": "66443924833011eeac3cb393f3d07f9f",
            "eventTime": "2023-11-14 20:57:19.186000000",
            "flagged": False,
            "loginName": "bob.ross@acme.com",
            "orgName": "acme.com",
            "requestUrl": "/access/v2/orgs/A1234567/grants",
            "verbose": False,
        },
    ),
]


class CarbonBlackAuditFlagged(PantherRule):
    RuleID = "CarbonBlack.Audit.Flagged-prototype"
    LogTypes = [PantherLogType.CarbonBlack_Audit]
    Description = "Detects when Carbon Black has flagged a log as important, such as failed login attempts and locked accounts."
    DisplayName = "Carbon Black Log Entry Flagged"
    Severity = PantherSeverity.High
    Tags = ["Credential Access", "Brute Force"]
    Reports = {"MITRE ATT&CK": ["TA0006:T1110"]}
    Reference = "https://docs.vmware.com/en/VMware-Carbon-Black-Cloud/services/carbon-black-cloud-user-guide/GUID-FB61E4E3-6431-4226-A4E3-5949FB75922B.html"
    Tests = carbon_black_audit_flagged_tests

    def rule(self, event):
        return event.get("flagged", False)

    def title(self, event):
        user = event.get("loginName", "<NO_USERNAME_FOUND>")
        ip_addr = event.get("clientIp", "<NO_IP_FOUND>")
        desc = event.get("description", "<NO_DESCRIPTION_FOUND>")
        return f"{user} [{ip_addr}] {desc}"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_box_helpers import (
    is_box_sdk_enabled,
    lookup_box_file,
    lookup_box_folder,
)
from pypanther.log_types import PantherLogType

box_item_shared_externally_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Regular Event",
        ExpectedResult=False,
        Log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {
                "id": 12345678,
                "type": "user",
                "login": "cat@example",
                "name": "Bob Cat",
            },
            "event_type": "DELETE",
            "source": {
                "item_name": "regular_file.pdf",
                "item_type": "file",
                "owned_by": {
                    "id": 12345678,
                    "type": "user",
                    "login": "cat@example",
                    "name": "Bob Cat",
                },
                "parent": {
                    "id": 12345,
                    "type": "folder",
                    "etag": 1,
                    "name": "Parent_Folder",
                    "sequence_id": 2,
                },
            },
        },
    )
]


class BoxItemSharedExternally(PantherRule):
    RuleID = "Box.Item.Shared.Externally-prototype"
    DisplayName = "Box item shared externally"
    Enabled = False
    LogTypes = [PantherLogType.Box_Event]
    Tags = ["Box", "Exfiltration:Exfiltration Over Web Service", "Configuration Required"]
    Reports = {"MITRE ATT&CK": ["TA0010:T1567"]}
    Severity = PantherSeverity.Medium
    Description = "A user has shared an item and it is accessible to anyone with the share link (internal or external to the company). This rule requires that the boxsdk[jwt] be installed in the environment.\n"
    Reference = "https://support.box.com/hc/en-us/articles/4404822772755-Enterprise-Settings-Content-Sharing-Tab"
    Runbook = "Investigate whether this user's activity is expected.\n"
    SummaryAttributes = ["ip_address"]
    Threshold = 10
    Tests = box_item_shared_externally_tests
    ALLOWED_SHARED_ACCESS = {"collaborators", "company"}
    SHARE_EVENTS = {
        "CHANGE_FOLDER_PERMISSION",
        "ITEM_SHARED",
        "ITEM_SHARED_CREATE",
        "ITEM_SHARED_UPDATE",
        "SHARE",
    }

    def rule(self, event):
        # filter events
        if event.get("event_type") not in self.SHARE_EVENTS:
            return False
        # only try to lookup file/folder info if sdk is enabled in the env
        if is_box_sdk_enabled():
            item = self.get_item(event)
            if item is not None and item.get("shared_link"):
                return (
                    deep_get(item, "shared_link", "effective_access")
                    not in self.ALLOWED_SHARED_ACCESS
                )
        return False

    def get_item(self, event):
        item_id = deep_get(event, "source", "item_id", default="")
        user_id = deep_get(event, "source", "owned_by", "id", default="")
        item = {}
        if deep_get(event, "source", "item_type") == "folder":
            item = lookup_box_folder(user_id, item_id)
        elif deep_get(event, "source", "item_type") == "file":
            item = lookup_box_file(user_id, item_id)
        return item

    def title(self, event):
        return f"User [{deep_get(event, 'created_by', 'login', default='<UNKNOWN_USER>')}] shared an item [{deep_get(event, 'source', 'item_name', default='<UNKNOWN_NAME>')}] externally."



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

okta_support_access_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Support Access Granted",
        ExpectedResult=True,
        Log={
            "published": "2022-03-22 14:21:53.225",
            "eventType": "user.session.impersonation.grant",
            "version": "0",
            "severity": "INFO",
            "legacyEventType": "core.user.impersonation.grant.enabled",
            "displayMessage": "Enable impersonation grant",
            "actor": {
                "alternateId": "homer@springfield.gov",
                "displayName": "Homer Simpson",
                "id": "111111",
                "type": "User",
            },
            "client": {
                "device": "Computer",
                "ipAddress": "1.1.1.1",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/99.0.4844.83 Safari/537.36",
                },
                "zone": "null",
            },
            "p_log_type": "Okta.SystemLog",
        },
    ),
    PantherRuleTest(
        Name="Login Event",
        ExpectedResult=False,
        Log={
            "published": "2022-03-22 14:21:53.225",
            "eventType": "user.session.start",
            "version": "0",
            "severity": "INFO",
            "actor": {
                "alternateId": "homer@springfield.gov",
                "displayName": "Homer Simpson",
                "id": "111111",
                "type": "User",
            },
            "client": {
                "device": "Computer",
                "ipAddress": "1.1.1.1",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/99.0.4844.83 Safari/537.36",
                },
                "zone": "null",
            },
            "p_log_type": "Okta.SystemLog",
        },
    ),
]


class OktaSupportAccess(PantherRule):
    RuleID = "Okta.Support.Access-prototype"
    DisplayName = "Okta Support Access Granted"
    LogTypes = [PantherLogType.Okta_SystemLog]
    Tags = [
        "Identity & Access Management",
        "DataModel",
        "Okta",
        "Initial Access:Trusted Relationship",
    ]
    Reports = {"MITRE ATT&CK": ["TA0001:T1199"]}
    Severity = PantherSeverity.Medium
    Description = "An admin user has granted access to Okta Support to your account"
    Reference = "https://help.okta.com/en/prod/Content/Topics/Settings/settings-support-access.htm"
    Runbook = "Contact Admin to ensure this was sanctioned activity"
    DedupPeriodMinutes = 15
    SummaryAttributes = ["eventType", "severity", "displayMessage", "p_any_ip_addresses"]
    Tests = okta_support_access_tests
    OKTA_SUPPORT_ACCESS_EVENTS = [
        "user.session.impersonation.grant",
        "user.session.impersonation.initiate",
    ]

    def rule(self, event):
        return event.get("eventType") in self.OKTA_SUPPORT_ACCESS_EVENTS

    def title(self, event):
        return f"Okta Support Access Granted by {event.udm('actor_user')}"

    def alert_context(self, event):
        context = {
            "user": event.udm("actor_user"),
            "ip": event.udm("source_ip"),
            "event": event.get("eventType"),
        }
        return context



from typing import List

from pypanther.base import PantherDataModel, PantherDataModelMapping
from pypanther.log_types import PantherLogType

# 1Password item usage logs don't have event types, this file is a placeholder. All events are
# the viewing or usage of an item in 1Password


class StandardOnePasswordItemUsage(PantherDataModel):
    DataModelID: str = "Standard.OnePassword.ItemUsage"
    DisplayName: str = "1Password Item Usage Events"
    Enabled: bool = True
    LogTypes: List[str] = [PantherLogType.OnePassword_ItemUsage]
    Mappings: List[PantherDataModelMapping] = [
        PantherDataModelMapping(Name="actor_user", Path="$.user.email"),
        PantherDataModelMapping(Name="source_ip", Path="$.client.ipaddress"),
    ]



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get, deep_walk, okta_alert_context
from pypanther.log_types import PantherLogType

okta_org2org_creation_modification_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Org2Org modified",
        ExpectedResult=True,
        Log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "geolocation": {"lat": 20, "lon": -25},
                    "postalCode": "12345",
                    "state": "Ohio",
                },
                "ipAddress": "1.3.2.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "device": {"name": "Evil Computer"},
            "debugcontext": {
                "debugData": {
                    "requestId": "AbCdEf12G",
                    "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                    "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                    "behaviors": {
                        "New Geo-Location=NEGATIVE": None,
                        "New Device=POSITIVE": None,
                        "New IP=POSITIVE": None,
                        "New State=NEGATIVE": None,
                        "New Country=NEGATIVE": None,
                        "Velocity=NEGATIVE": None,
                        "New City=NEGATIVE": None,
                    },
                }
            },
            "displaymessage": "Evaluation of sign-on policy",
            "eventtype": "application.lifecycle.update",
            "outcome": {
                "reason": "Sign-on policy evaluation resulted in CHALLENGE",
                "result": "CHALLENGE",
            },
            "published": "2022-06-22 18:18:29.015",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Springfield",
                            "country": "United States",
                            "geolocation": {"lat": 20, "lon": -25},
                            "postalCode": "12345",
                            "state": "Ohio",
                            "ip": "1.3.2.4",
                            "version": "V4",
                        }
                    }
                ]
            },
            "securitycontext": {
                "asNumber": 701,
                "asOrg": "verizon",
                "domain": "verizon.net",
                "isProxy": False,
                "isp": "verizon",
            },
            "severity": "INFO",
            "target": [
                {
                    "alternateId": "Okta Org2Org",
                    "displayName": "Okta Org2Org",
                    "type": "AppInstance",
                },
                {
                    "alternateId": "peter.griffin@company.com",
                    "displayName": "Peter Griffin",
                    "id": "0002222AAAA",
                    "type": "User",
                },
            ],
            "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
            "uuid": "AbC-123-XyZ",
            "version": "0",
        },
    ),
    PantherRuleTest(
        Name="Org2Org created",
        ExpectedResult=True,
        Log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "geolocation": {"lat": 20, "lon": -25},
                    "postalCode": "12345",
                    "state": "Ohio",
                },
                "ipAddress": "1.3.2.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "device": {"name": "Evil Computer"},
            "debugcontext": {
                "debugData": {
                    "requestId": "AbCdEf12G",
                    "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                    "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                    "logOnlySecurityData": {
                        "risk": {"level": "LOW"},
                        "behaviors": {
                            "New Geo-Location": "NEGATIVE",
                            "New Device": "POSITIVE",
                            "New IP": "POSITIVE",
                            "New State": "NEGATIVE",
                            "New Country": "NEGATIVE",
                            "Velocity": "NEGATIVE",
                            "New City": "NEGATIVE",
                        },
                    },
                }
            },
            "displaymessage": "Evaluation of sign-on policy",
            "eventtype": "application.lifecycle.create",
            "outcome": {
                "reason": "Sign-on policy evaluation resulted in CHALLENGE",
                "result": "CHALLENGE",
            },
            "published": "2022-06-22 18:18:29.015",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Springfield",
                            "country": "United States",
                            "geolocation": {"lat": 20, "lon": -25},
                            "postalCode": "12345",
                            "state": "Ohio",
                            "ip": "1.3.2.4",
                            "version": "V4",
                        }
                    }
                ]
            },
            "securitycontext": {
                "asNumber": 701,
                "asOrg": "verizon",
                "domain": "verizon.net",
                "isProxy": False,
                "isp": "verizon",
            },
            "severity": "INFO",
            "target": [
                {
                    "alternateId": "Random Org2Org",
                    "displayName": "Random Org2Org",
                    "type": "AppInstance",
                },
                {
                    "alternateId": "peter.griffin@company.com",
                    "displayName": "Peter Griffin",
                    "id": "0002222AAAA",
                    "type": "User",
                },
            ],
            "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
            "uuid": "AbC-123-XyZ",
            "version": "0",
        },
    ),
    PantherRuleTest(
        Name="Not New Behavior",
        ExpectedResult=False,
        Log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "geolocation": {"lat": 20, "lon": -25},
                    "postalCode": "12345",
                    "state": "Ohio",
                },
                "ipAddress": "1.3.2.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "requestId": "AbCdEf12G",
                    "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                    "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                    "logOnlySecurityData": {
                        "risk": {"level": "LOW"},
                        "behaviors": {
                            "New Geo-Location": "NEGATIVE",
                            "New Device": "NEGATIVE",
                            "New IP": "NEGATIVE",
                            "New State": "NEGATIVE",
                            "New Country": "NEGATIVE",
                            "Velocity": "NEGATIVE",
                            "New City": "NEGATIVE",
                        },
                    },
                }
            },
            "displaymessage": "Evaluation of sign-on policy",
            "eventtype": "policy.evaluate_sign_on",
            "outcome": {
                "reason": "Sign-on policy evaluation resulted in CHALLENGE",
                "result": "CHALLENGE",
            },
            "published": "2022-06-22 18:18:29.015",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Springfield",
                            "country": "United States",
                            "geolocation": {"lat": 20, "lon": -25},
                            "postalCode": "12345",
                            "state": "Ohio",
                            "ip": "1.3.2.4",
                            "version": "V4",
                        }
                    }
                ]
            },
            "securitycontext": {
                "asNumber": 701,
                "asOrg": "verizon",
                "domain": "verizon.net",
                "isProxy": False,
                "isp": "verizon",
            },
            "severity": "INFO",
            "target": [
                {
                    "alternateId": "Okta Admin Console",
                    "displayName": "Okta Admin Console",
                    "type": "AppInstance",
                },
                {
                    "alternateId": "peter.griffin@company.com",
                    "displayName": "Peter Griffin",
                    "id": "0002222AAAA",
                    "type": "User",
                },
            ],
            "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
            "uuid": "AbC-123-XyZ",
            "version": "0",
        },
    ),
]


class OktaOrg2orgCreationModification(PantherRule):
    RuleID = "Okta.Org2org.Creation.Modification-prototype"
    DisplayName = "Okta Org2Org application created of modified"
    LogTypes = [PantherLogType.Okta_SystemLog]
    Reports = {"MITRE ATT&CK": ["TA0006:T1556", "TA0004:T1078.004"]}
    Severity = PantherSeverity.High
    Description = "An Okta Org2Org application has been created or modified. Okta's Org2Org applications instances are used to push and match users from one Okta organization to another. A malicious actor can add an Org2Org application instance and create a user in the source organization (controlled by the attacker) with the same identifier as a Super Administrator in the target organization.\n"
    Reference = "https://www.authomize.com/blog/authomize-discovers-password-stealing-and-impersonation-risks-to-in-okta/\n"
    Tests = okta_org2org_creation_modification_tests
    APP_LIFECYCLE_EVENTS = (
        "application.lifecycle.update",
        "application.lifecycle.create",
        "application.lifecycle.activate",
    )

    def rule(self, event):
        if event.get("eventType") not in self.APP_LIFECYCLE_EVENTS:
            return False
        return "Org2Org" in deep_walk(
            event, "target", "displayName", default="", return_val="first"
        )

    def title(self, event):
        action = event.get("eventType").split(".")[-1]
        target = deep_walk(
            event, "target", "alternateId", default="<alternateId-not-found>", return_val="first"
        )
        return f"{deep_get(event, 'actor', 'displayName', default='<displayName-not-found>')} <{deep_get(event, 'actor', 'alternateId', default='alternateId-not-found')}> {action}d Org2Org app [{target}]"

    def severity(self, event):
        if "create" in event.get("eventType"):
            return "HIGH"
        return "MEDIUM"

    def alert_context(self, event):
        return okta_alert_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import aws_rule_context
from pypanther.log_types import PantherLogType

aws_security_hub_finding_evasion_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="CreateInsight",
        ExpectedResult=False,
        Log={
            "awsRegion": "us-west-2",
            "eventID": "3dabcebf-35b0-443f-a1a2-26e186ce23bf",
            "eventName": "CreateInsight",
            "eventSource": "securityhub.amazonaws.com",
            "eventTime": "2018-11-25T01:02:18Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "readOnly": False,
            "recipientAccountId": "012345678901",
            "requestID": "c0fffccd-f04d-11e8-93fc-ddcd14710066",
            "requestParameters": {
                "Filters": {},
                "Name": "Test Insight",
                "ResultField": "ResourceId",
            },
            "responseElements": {
                "InsightArn": "arn:aws:securityhub:us-west-2:0123456789010:insight/custom/f4c4890b-ac6b-4c26-95f9-e62cc46f3055"
            },
            "sourceIPAddress": "205.251.233.179",
            "userAgent": "aws-cli/1.11.76 Python/2.7.10 Darwin/17.7.0 botocore/1.5.39",
            "userIdentity": {
                "accessKeyId": "AKIAIOSFODNN7EXAMPLE",
                "accountId": "012345678901",
                "arn": "arn:aws:iam::012345678901:user/TestUser",
                "principalId": "AIDAJK6U5DS22IAVUI7BW",
                "type": "IAMUser",
                "userName": "TestUser",
            },
        },
    ),
    PantherRuleTest(
        Name="DeleteInsight",
        ExpectedResult=True,
        Log={
            "awsRegion": "us-west-2",
            "eventID": "3dabcebf-35b0-443f-a1a2-26e186ce23bf",
            "eventName": "DeleteInsight",
            "eventSource": "securityhub.amazonaws.com",
            "eventTime": "2018-11-25T01:02:18Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "readOnly": False,
            "recipientAccountId": "012345678901",
            "requestID": "c0fffccd-f04d-11e8-93fc-ddcd14710066",
            "requestParameters": {
                "Filters": {},
                "Name": "Test Insight",
                "ResultField": "ResourceId",
            },
            "responseElements": {
                "InsightArn": "arn:aws:securityhub:us-west-2:0123456789010:insight/custom/f4c4890b-ac6b-4c26-95f9-e62cc46f3055"
            },
            "sourceIPAddress": "205.251.233.179",
            "userAgent": "aws-cli/1.11.76 Python/2.7.10 Darwin/17.7.0 botocore/1.5.39",
            "userIdentity": {
                "accessKeyId": "AKIAIOSFODNN7EXAMPLE",
                "accountId": "012345678901",
                "arn": "arn:aws:iam::012345678901:user/TestUser",
                "principalId": "AIDAJK6U5DS22IAVUI7BW",
                "type": "IAMUser",
                "userName": "TestUser",
            },
        },
    ),
    PantherRuleTest(
        Name="UpdateFindings",
        ExpectedResult=True,
        Log={
            "awsRegion": "us-west-2",
            "eventID": "3dabcebf-35b0-443f-a1a2-26e186ce23bf",
            "eventName": "UpdateFindings",
            "eventSource": "securityhub.amazonaws.com",
            "eventTime": "2018-11-25T01:02:18Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "readOnly": False,
            "recipientAccountId": "012345678901",
            "requestID": "c0fffccd-f04d-11e8-93fc-ddcd14710066",
            "requestParameters": {
                "Filters": {},
                "Name": "Test Insight",
                "ResultField": "ResourceId",
            },
            "responseElements": {
                "InsightArn": "arn:aws:securityhub:us-west-2:0123456789010:insight/custom/f4c4890b-ac6b-4c26-95f9-e62cc46f3055"
            },
            "sourceIPAddress": "205.251.233.179",
            "userAgent": "aws-cli/1.11.76 Python/2.7.10 Darwin/17.7.0 botocore/1.5.39",
            "userIdentity": {
                "accessKeyId": "AKIAIOSFODNN7EXAMPLE",
                "accountId": "012345678901",
                "arn": "arn:aws:iam::012345678901:user/TestUser",
                "principalId": "AIDAJK6U5DS22IAVUI7BW",
                "type": "IAMUser",
                "userName": "TestUser",
            },
        },
    ),
]


class AWSSecurityHubFindingEvasion(PantherRule):
    Description = "Detections modification of findings in SecurityHub"
    DisplayName = "AWS SecurityHub Finding Evasion"
    Reports = {"MITRE ATT&CK": ["TA0005:T1562"]}
    Reference = "https://docs.aws.amazon.com/securityhub/latest/userguide/securityhub-insights-view-take-action.html"
    Severity = PantherSeverity.High
    LogTypes = [PantherLogType.AWS_CloudTrail]
    RuleID = "AWS.SecurityHub.Finding.Evasion-prototype"
    Tests = aws_security_hub_finding_evasion_tests
    EVASION_OPERATIONS = ["BatchUpdateFindings", "DeleteInsight", "UpdateFindings", "UpdateInsight"]

    def rule(self, event):
        if (
            event.get("eventSource", "") == "securityhub.amazonaws.com"
            and event.get("eventName", "") in self.EVASION_OPERATIONS
        ):
            return True
        return False

    def title(self, event):
        return f"SecurityHub Findings have been modified in account: [{event.get('recipientAccountId', '')}]"

    def alert_context(self, event):
        return aws_rule_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_duo_helpers import duo_alert_context
from pypanther.log_types import PantherLogType

duo_admin_mfa_restrictions_updated_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Admin MFA Update Event",
        ExpectedResult=True,
        Log={
            "action": "update_admin_factor_restrictions",
            "description": '{"allowed_factors": "Duo mobile passcodes, Hardware tokens, Duo push, Yubikey aes"}',
            "isotimestamp": "2022-02-21 21:48:06",
            "timestamp": "2022-02-21 21:48:06",
            "username": "Homer Simpson",
        },
    ),
    PantherRuleTest(
        Name="Login Event",
        ExpectedResult=False,
        Log={
            "action": "admin_login",
            "description": '{"ip_address": "1.2.3.4", "device": "123-456-789", "factor": "sms", "saml_idp": "OneLogin", "primary_auth_method": "Single Sign-On"}',
            "isotimestamp": "2021-06-30 19:45:37",
            "timestamp": "2021-06-30 19:45:37",
            "username": "Homer Simpson",
        },
    ),
]


class DuoAdminMFARestrictionsUpdated(PantherRule):
    Description = (
        "Detects changes to allowed MFA factors administrators can use to log into the admin panel."
    )
    DisplayName = "Duo Admin MFA Restrictions Updated"
    Reference = "https://duo.com/docs/essentials-overview"
    Severity = PantherSeverity.Medium
    LogTypes = [PantherLogType.Duo_Administrator]
    RuleID = "Duo.Admin.MFA.Restrictions.Updated-prototype"
    Tests = duo_admin_mfa_restrictions_updated_tests

    def rule(self, event):
        return event.get("action") == "update_admin_factor_restrictions"

    def title(self, event):
        return (
            f"Duo Admin MFA Restrictions Updated by [{event.get('username', '<user_not_found>')}]"
        )

    def alert_context(self, event):
        return duo_alert_context(event)



from typing import List

from pypanther.base import PantherDataModel, PantherDataModelMapping
from pypanther.log_types import PantherLogType


class StandardAWSVPCFlow(PantherDataModel):
    DataModelID: str = "Standard.AWS.VPCFlow"
    DisplayName: str = "AWS VPCFlow"
    Enabled: bool = True
    LogTypes: List[str] = [PantherLogType.AWS_VPCFlow]
    Mappings: List[PantherDataModelMapping] = [
        PantherDataModelMapping(Name="destination_ip", Path="dstAddr"),
        PantherDataModelMapping(Name="destination_port", Path="dstPort"),
        PantherDataModelMapping(Name="source_ip", Path="srcAddr"),
        PantherDataModelMapping(Name="source_port", Path="srcPort"),
        PantherDataModelMapping(Name="user_agent", Path="userAgent"),
        PantherDataModelMapping(Name="log_status", Path="log-status"),
    ]



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_mongodb_helpers import mongodb_alert_context
from pypanther.log_types import PantherLogType

mongo_db_identity_provider_activity_tests: List[PantherRuleTest] = [
    PantherRuleTest(Name="Random event", ExpectedResult=False, Log={"eventTypeName": "cat_jumped"}),
    PantherRuleTest(
        Name="FEDERATION_SETTINGS_CREATED",
        ExpectedResult=True,
        Log={"eventTypeName": "FEDERATION_SETTINGS_CREATED"},
    ),
    PantherRuleTest(
        Name="IDENTITY_PROVIDER_CREATED",
        ExpectedResult=True,
        Log={"eventTypeName": "IDENTITY_PROVIDER_CREATED"},
    ),
]


class MongoDBIdentityProviderActivity(PantherRule):
    Description = "Changes to identity provider settings are privileged activities that should be carefully audited.  Attackers may add or change IDP integrations to gain persistence to environments"
    DisplayName = "MongoDB Identity Provider Activity"
    Severity = PantherSeverity.Medium
    Reference = "https://attack.mitre.org/techniques/T1556/007/"
    LogTypes = [PantherLogType.MongoDB_OrganizationEvent]
    RuleID = "MongoDB.Identity.Provider.Activity-prototype"
    Tests = mongo_db_identity_provider_activity_tests

    def rule(self, event):
        important_event_types = {
            "FEDERATION_SETTINGS_CREATED",
            "FEDERATION_SETTINGS_DELETED",
            "FEDERATION_SETTINGS_UPDATED",
            "IDENTITY_PROVIDER_CREATED",
            "IDENTITY_PROVIDER_UPDATED",
            "IDENTITY_PROVIDER_DELETED",
            "IDENTITY_PROVIDER_ACTIVATED",
            "IDENTITY_PROVIDER_DEACTIVATED",
            "IDENTITY_PROVIDER_JWKS_REVOKED",
            "OIDC_IDENTITY_PROVIDER_UPDATED",
            "OIDC_IDENTITY_PROVIDER_ENABLED",
            "OIDC_IDENTITY_PROVIDER_DISABLED",
        }
        return event.deep_get("eventTypeName") in important_event_types

    def title(self, event):
        target_username = event.get("targetUsername", "<USER_NOT_FOUND>")
        org_id = event.get("orgId", "<ORG_NOT_FOUND>")
        return f"MongoDB Atlas: User [{target_username}] roles changed in org [{org_id}]"

    def alert_context(self, event):
        return mongodb_alert_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_notion_helpers import notion_alert_context
from pypanther.log_types import PantherLogType

notion_audit_log_exported_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Other Event",
        ExpectedResult=False,
        Log={
            "event": {
                "id": "...",
                "timestamp": "2023-05-15T19:14:21.031Z",
                "workspace_id": "..",
                "actor": {
                    "id": "..",
                    "object": "user",
                    "type": "person",
                    "person": {"email": "homer.simpson@yourcompany.io"},
                },
                "ip_address": "...",
                "platform": "web",
                "type": "workspace.content_exported",
                "workspace.content_exported": {},
            }
        },
    ),
    PantherRuleTest(
        Name="Audit Log Exported",
        ExpectedResult=True,
        Log={
            "event": {
                "id": "...",
                "timestamp": "2023-05-15T19:14:21.031Z",
                "workspace_id": "..",
                "actor": {
                    "object": "user",
                    "id": "..",
                    "type": "person",
                    "person": {"email": "homer.simpson@yourcompany.io"},
                },
                "ip_address": "...",
                "platform": "web",
                "type": "workspace.audit_log_exported",
                "details": {"duration_in_days": 30},
            }
        },
    ),
]


class NotionAuditLogExported(PantherRule):
    RuleID = "Notion.Audit.Log.Exported-prototype"
    DisplayName = "Notion Audit Log Exported"
    LogTypes = [PantherLogType.Notion_AuditLogs]
    Tags = ["Notion", "Data Security", "Data Exfiltration"]
    Severity = PantherSeverity.Medium
    Description = "A Notion User exported audit logs for your organization’s workspace."
    Runbook = "Possible Data Exfiltration. Follow up with the Notion User to determine if this was done for a valid business reason."
    Reference = "https://www.notion.so/help/audit-log#export-your-audit-log"
    Tests = notion_audit_log_exported_tests

    def rule(self, event):
        event_type = event.deep_get("event", "type", default="<NO_EVENT_TYPE_FOUND>")
        return event_type == "workspace.audit_log_exported"

    def title(self, event):
        user = event.deep_get("event", "actor", "person", "email", default="<NO_USER_FOUND>")
        workspace_id = event.deep_get("event", "workspace_id", default="<NO_WORKSPACE_ID_FOUND>")
        duration_in_days = event.deep_get(
            "event", "details", "duration_in_days", default="<NO_DURATION_IN_DAYS_FOUND>"
        )
        return f"Notion User [{user}] exported audit logs for the last {duration_in_days} days for workspace id {workspace_id}"

    def alert_context(self, event):
        return notion_alert_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_config import config
from pypanther.log_types import PantherLogType

g_suite_doc_ownership_transfer_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Ownership Transferred Within Organization",
        ExpectedResult=False,
        Log={
            "id": {"applicationName": "admin"},
            "name": "TRANSFER_DOCUMENT_OWNERSHIP",
            "parameters": {"NEW_VALUE": "homer.simpson@example.com"},
        },
    ),
    PantherRuleTest(
        Name="Document Transferred to External User",
        ExpectedResult=True,
        Log={
            "id": {"applicationName": "admin"},
            "name": "TRANSFER_DOCUMENT_OWNERSHIP",
            "parameters": {"NEW_VALUE": "monty.burns@badguy.com"},
        },
    ),
]


class GSuiteDocOwnershipTransfer(PantherRule):
    RuleID = "GSuite.DocOwnershipTransfer-prototype"
    DisplayName = "GSuite Document External Ownership Transfer"
    Enabled = False
    LogTypes = [PantherLogType.GSuite_ActivityEvent]
    Tags = ["GSuite", "Configuration Required", "Collection:Data from Information Repositories"]
    Reports = {"MITRE ATT&CK": ["TA0009:T1213"]}
    Severity = PantherSeverity.Low
    Description = "A GSuite document's ownership was transferred to an external party.\n"
    Reference = "https://support.google.com/drive/answer/2494892?hl=en&co=GENIE.Platform%3DDesktop&sjid=864417124752637253-EU"
    Runbook = (
        "Verify that this document did not contain sensitive or private company information.\n"
    )
    SummaryAttributes = ["actor:email"]
    Tests = g_suite_doc_ownership_transfer_tests
    GSUITE_TRUSTED_OWNERSHIP_DOMAINS = {
        "@" + domain for domain in config.GSUITE_TRUSTED_OWNERSHIP_DOMAINS
    }

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "admin":
            return False
        if bool(event.get("name") == "TRANSFER_DOCUMENT_OWNERSHIP"):
            new_owner = deep_get(event, "parameters", "NEW_VALUE", default="<UNKNOWN USER>")
            return bool(new_owner) and (
                not any((new_owner.endswith(x) for x in self.GSUITE_TRUSTED_OWNERSHIP_DOMAINS))
            )
        return False



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import okta_alert_context
from pypanther.log_types import PantherLogType

okta_group_admin_role_assigned_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Group Privilege Grant",
        ExpectedResult=True,
        Log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpsons",
                "id": "00ABC123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "xyz1234"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "geolocation": {"lat": 11.111, "lon": -70},
                    "postalCode": "1234",
                    "state": "California",
                },
                "ipAddress": "1.2.3.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "authnRequestId": "ABC123",
                    "deviceFingerprint": "009988771ABC",
                    "dtHash": "123abc1234",
                    "requestId": "abc-111-adf",
                    "requestUri": "/idp/idx/identify",
                    "threatSuspected": "false",
                    "url": "/idp/idx/identify?",
                }
            },
            "displaymessage": "Group Privilege granted",
            "eventtype": "group.privilege.grant",
            "legacyeventtype": "group.privilege.grant",
            "outcome": {"result": "FAILURE"},
            "published": "2022-12-13 00:58:19.811",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Springfield",
                            "country": "United States",
                            "geolocation": {"lat": 11.111, "lon": -70},
                            "postalCode": "1234",
                            "state": "California",
                        },
                        "ip": "1.2.3.4",
                        "version": "V4",
                    }
                ]
            },
            "securitycontext": {
                "asNumber": 11351,
                "asOrg": "charter communications inc",
                "domain": "rr.com",
                "isProxy": False,
                "isp": "charter communications inc",
            },
            "severity": "WARN",
            "target": [
                {
                    "alternateId": "App (123)",
                    "displayName": "App (123)",
                    "id": "12345",
                    "type": "AppInstance",
                }
            ],
            "transaction": {"detail": {}, "id": "aaa-bbb-123", "type": "WEB"},
            "uuid": "aa-11-22-33-44-bb",
            "version": "0",
        },
    ),
    PantherRuleTest(
        Name="Non Event",
        ExpectedResult=False,
        Log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpsons",
                "id": "00ABC123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "xyz1234"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "geolocation": {"lat": 11.111, "lon": -70},
                    "postalCode": "1234",
                    "state": "California",
                },
                "ipAddress": "1.2.3.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "authnRequestId": "ABC123",
                    "deviceFingerprint": "009988771ABC",
                    "dtHash": "123abc1234",
                    "requestId": "abc-111-adf",
                    "requestUri": "/idp/idx/identify",
                    "threatSuspected": "false",
                    "url": "/idp/idx/identify?",
                }
            },
            "displaymessage": "User attempted to reuse tokens",
            "eventtype": "app.token.reuse",
            "legacyeventtype": "app.token.reuse",
            "outcome": {"result": "FAILURE"},
            "published": "2022-12-13 00:58:19.811",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Springfield",
                            "country": "United States",
                            "geolocation": {"lat": 11.111, "lon": -70},
                            "postalCode": "1234",
                            "state": "California",
                        },
                        "ip": "1.2.3.4",
                        "version": "V4",
                    }
                ]
            },
            "securitycontext": {
                "asNumber": 11351,
                "asOrg": "charter communications inc",
                "domain": "rr.com",
                "isProxy": False,
                "isp": "charter communications inc",
            },
            "severity": "WARN",
            "target": [
                {
                    "alternateId": "App (123)",
                    "displayName": "App (123)",
                    "id": "12345",
                    "type": "AppInstance",
                }
            ],
            "transaction": {"detail": {}, "id": "aaa-bbb-123", "type": "WEB"},
            "uuid": "aa-11-22-33-44-bb",
            "version": "0",
        },
    ),
]


class OktaGroupAdminRoleAssigned(PantherRule):
    Description = "Detect when an admin role is assigned to a group"
    DisplayName = "Okta Group Admin Role Assigned"
    Reference = "https://support.okta.com/help/s/article/How-to-assign-Administrator-roles-to-groups?language=en_US#:~:text=Log%20in%20to%20the%20Admin,user%20and%20click%20Save%20changes"
    Severity = PantherSeverity.High
    LogTypes = [PantherLogType.Okta_SystemLog]
    RuleID = "Okta.Group.Admin.Role.Assigned-prototype"
    Tests = okta_group_admin_role_assigned_tests

    def rule(self, event):
        return event.get("eventtype", "") == "group.privilege.grant"

    def title(self, event):
        # pylint: disable=W0613
        return f"Okta Admin Privileges Assigned to Group [{event.get('target', [{}])[0].get('alternateId', '<id-not-found>')}]"

    def alert_context(self, event):
        return okta_alert_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_duo_helpers import (
    deserialize_administrator_log_event_description,
    duo_alert_context,
)
from pypanther.log_types import PantherLogType

duo_admin_create_admin_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Admin Create",
        ExpectedResult=True,
        Log={
            "action": "admin_create",
            "description": '{"name": "Homer Simpson", "phone": null, "is_temporary_password": false, "email": "homer.simpson@simpsons.com", "hardtoken": null, "role": "Owner", "status": "Pending Activation", "restricted_by_admin_units": false, "administrative_units": ""}',
            "isotimestamp": "2023-01-17 16:47:54",
            "object": "Homer Simpson",
            "timestamp": "2023-01-17 16:47:54",
            "username": "Bart Simpson",
        },
    ),
    PantherRuleTest(
        Name="Other Event",
        ExpectedResult=False,
        Log={
            "action": "admin_login",
            "description": '{"ip_address": "1.2.3.4", "device": "123-456-123", "factor": "sms", "saml_idp": "OneLogin", "primary_auth_method": "Single Sign-On"}',
            "isotimestamp": "2021-07-02 18:31:25",
            "timestamp": "2021-07-02 18:31:25",
            "username": "Homer Simpson",
        },
    ),
]


class DuoAdminCreateAdmin(PantherRule):
    Description = "A new Duo Administrator was created. "
    DisplayName = "Duo Admin Create Admin"
    Reference = "https://duo.com/docs/administration-admins#add-an-administrator"
    Severity = PantherSeverity.High
    LogTypes = [PantherLogType.Duo_Administrator]
    RuleID = "Duo.Admin.Create.Admin-prototype"
    Tests = duo_admin_create_admin_tests

    def rule(self, event):
        return event.get("action") == "admin_create"

    def title(self, event):
        event_description = deserialize_administrator_log_event_description(event)
        return f"Duo: [{event.get('username', '<username_not_found>')}] created a new admin account: [{event_description.get('name', '<name_not_found>')}] [{event_description.get('email', '<email_not_found>')}]."

    def alert_context(self, event):
        return duo_alert_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_notion_helpers import notion_alert_context
from pypanther.log_types import PantherLogType

notion_samlsso_configuration_changed_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Other Event",
        ExpectedResult=False,
        Log={
            "event": {
                "id": "...",
                "timestamp": "2023-05-15T19:14:21.031Z",
                "workspace_id": "..",
                "actor": {
                    "id": "..",
                    "object": "user",
                    "type": "person",
                    "person": {"email": "homer.simpson@yourcompany.io"},
                },
                "ip_address": "...",
                "platform": "web",
                "type": "workspace.content_exported",
                "workspace.content_exported": {},
            }
        },
    ),
    PantherRuleTest(
        Name="SAML SSO Enabled",
        ExpectedResult=True,
        Log={
            "event": {
                "id": "...",
                "timestamp": "2023-05-15T19:14:21.031Z",
                "workspace_id": "..",
                "actor": {
                    "id": "..",
                    "object": "user",
                    "type": "person",
                    "person": {"email": "homer.simpson@yourcompany.io"},
                },
                "ip_address": "...",
                "platform": "web",
                "type": "workspace.settings.enforce_saml_sso_config_updated",
                "workspace.settings.enforce_saml_sso_config_updated": {"state": "enabled"},
            }
        },
    ),
    PantherRuleTest(
        Name="SAML SSO Disabled",
        ExpectedResult=True,
        Log={
            "event": {
                "id": "...",
                "timestamp": "2023-05-15T19:14:21.031Z",
                "workspace_id": "..",
                "actor": {
                    "id": "..",
                    "object": "user",
                    "type": "person",
                    "person": {"email": "homer.simpson@yourcompany.io"},
                },
                "ip_address": "...",
                "platform": "web",
                "type": "workspace.settings.enforce_saml_sso_config_updated",
                "workspace.settings.enforce_saml_sso_config_updated": {"state": "disabled"},
            }
        },
    ),
]


class NotionSAMLSSOConfigurationChanged(PantherRule):
    RuleID = "Notion.SAML.SSO.Configuration.Changed-prototype"
    DisplayName = "Notion SAML SSO Configuration Changed"
    LogTypes = [PantherLogType.Notion_AuditLogs]
    Tags = ["Notion", "Identity & Access Management", "Credential Security"]
    Severity = PantherSeverity.High
    Description = (
        "A Notion User changed settings to enforce SAML SSO configurations for your organization."
    )
    Runbook = "Follow up with the Notion User to determine if this was done for a valid business reason and to ensure these settings get re-enabled quickly for best security practices."
    Reference = "https://www.notion.so/help/saml-sso-configuration"
    Tests = notion_samlsso_configuration_changed_tests

    def rule(self, event):
        return (
            event.deep_get("event", "type", default="<NO_EVENT_TYPE_FOUND>")
            == "workspace.settings.enforce_saml_sso_config_updated"
        )

    def title(self, event):
        user = event.deep_get("event", "actor", "person", "email", default="<NO_USER_FOUND>")
        workspace_id = event.deep_get("event", "workspace_id", default="<NO_WORKSPACE_ID_FOUND>")
        state = deep_get(
            event,
            "event",
            "workspace.settings.enforce_saml_sso_config_updated",
            "state",
            default="<NO_STATE_FOUND>",
        )
        if state == "enabled":
            return f"Notion User [{user}] updated settings to enable SAML SSO config from workspace id {workspace_id}"
        return f"Notion User [{user}] updated settings to disable SAML SSO config from workspace id {workspace_id}"

    def severity(self, event):
        state = deep_get(
            event,
            "event",
            "workspace.settings.enforce_saml_sso_config_updated",
            "state",
            default="<NO_STATE_FOUND>",
        )
        if state == "enabled":
            return "INFO"
        return "HIGH"

    def alert_context(self, event):
        return notion_alert_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import aws_rule_context
from pypanther.log_types import PantherLogType

awsec2_ebs_encryption_disabled_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="DisableEbsEncryptionByDefault Event",
        ExpectedResult=True,
        Log={
            "awsRegion": "us-east-1",
            "eventName": "DisableEbsEncryptionByDefault",
            "eventSource": "ec2.amazonaws.com",
            "recipientAccountId": "123456789",
            "sourceIPAddress": "1.2.3.4",
            "userAgent": "Chrome Browser",
        },
    ),
    PantherRuleTest(
        Name="Non Matching Event",
        ExpectedResult=False,
        Log={
            "awsRegion": "ap-northeast-1",
            "eventName": "DescribeInstanceStatus",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2022-09-25 16:16:37",
            "eventType": "AwsApiCall",
            "readOnly": True,
            "sourceIPAddress": "1.2.3.4",
            "userAgent": "Datadog",
        },
    ),
]


class AWSEC2EBSEncryptionDisabled(PantherRule):
    Description = "Identifies disabling of default EBS encryption. Disabling default encryption does not change the encryption status of existing volumes. "
    DisplayName = "AWS EC2 EBS Encryption Disabled"
    Reports = {"MITRE ATT&CK": ["TA0040:T1486", "TA0040:T1565"]}
    Runbook = (
        "Verify this action was intended and if any EBS volumes were created after the change."
    )
    Reference = "https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html#encryption-by-default"
    Severity = PantherSeverity.Medium
    LogTypes = [PantherLogType.AWS_CloudTrail]
    RuleID = "AWS.EC2.EBS.Encryption.Disabled-prototype"
    Tests = awsec2_ebs_encryption_disabled_tests

    def rule(self, event):
        return (
            event.get("eventSource") == "ec2.amazonaws.com"
            and event.get("eventName") == "DisableEbsEncryptionByDefault"
        )

    def title(self, event):
        return f"EC2 EBS Default Encryption was disabled in [{event.get('recipientAccountId')}] - [{event.get('awsRegion')}]"

    def alert_context(self, event):
        return aws_rule_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

g_suite_two_step_verification_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Two Step Verification Enabled",
        ExpectedResult=False,
        Log={
            "id": {"applicationName": "user_accounts"},
            "actor": {"callerType": "USER", "email": "some.user@somedomain.com"},
            "kind": "admin#reports#activity",
            "type": "2sv_change",
            "name": "2sv_enroll",
        },
    ),
    PantherRuleTest(
        Name="Two Step Verification Disabled",
        ExpectedResult=True,
        Log={
            "id": {"applicationName": "user_accounts"},
            "actor": {"callerType": "USER", "email": "some.user@somedomain.com"},
            "kind": "admin#reports#activity",
            "type": "2sv_change",
            "name": "2sv_disable",
        },
    ),
]


class GSuiteTwoStepVerification(PantherRule):
    RuleID = "GSuite.TwoStepVerification-prototype"
    DisplayName = "GSuite User Two Step Verification Change"
    LogTypes = [PantherLogType.GSuite_ActivityEvent]
    Tags = ["GSuite", "Defense Evasion:Modify Authentication Process"]
    Reports = {"MITRE ATT&CK": ["TA0005:T1556"]}
    Severity = PantherSeverity.Low
    Description = "A user disabled two step verification for themselves.\n"
    Reference = "https://support.google.com/mail/answer/185839?hl=en&co=GENIE.Platform%3DDesktop&sjid=864417124752637253-EU"
    Runbook = "Depending on company policy, either suggest or require the user re-enable two step verification.\n"
    SummaryAttributes = ["actor:email"]
    Tests = g_suite_two_step_verification_tests

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "user_accounts":
            return False
        if event.get("type") == "2sv_change" and event.get("name") == "2sv_disable":
            return True
        return False

    def title(self, event):
        return f"Two step verification was disabled for user [{deep_get(event, 'actor', 'email', default='<UNKNOWN_USER>')}]"



from typing import List

from pypanther.base import PantherDataModel, PantherDataModelMapping
from pypanther.log_types import PantherLogType


def get_dns_query(event):
    # Strip trailing period.
    # Domain Names from Cisco Umbrella end with a trailing period, such as google.com.
    domain = event.get("domain")
    if domain:
        domain = domain.rstrip(".")
    return domain


class StandardCiscoUmbrellaDNS(PantherDataModel):
    DataModelID: str = "Standard.CiscoUmbrella.DNS"
    DisplayName: str = "Cisco Umbrella DNS"
    Enabled: bool = True
    LogTypes: List[str] = [PantherLogType.CiscoUmbrella_DNS]
    Mappings: List[PantherDataModelMapping] = [
        PantherDataModelMapping(Name="source_ip", Path="internalIp"),
        PantherDataModelMapping(Name="source_port", Path="srcPort"),
        PantherDataModelMapping(Name="dns_query", Method=get_dns_query),
    ]



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.gcp_base_helpers import gcp_alert_context
from pypanther.helpers.panther_base_helpers import deep_get, deep_walk
from pypanther.log_types import PantherLogType

gcpgke_kubernetes_cron_job_created_or_modified_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="create",
        ExpectedResult=True,
        Log={
            "protoPayload": {
                "authorizationInfo": [
                    {"granted": True, "permission": "io.k8s.batch.v1.cronjobs.create"}
                ],
                "methodName": "v2.deploymentmanager.deployments.insert",
                "serviceName": "deploymentmanager.googleapis.com",
            },
            "receiveTimestamp": "2024-01-19 13:47:19.465856238",
            "resource": {
                "labels": {"name": "test-vm-deployment", "project_id": "panther-threat-research"},
                "type": "deployment",
            },
            "severity": "NOTICE",
            "timestamp": "2024-01-19 13:47:18.279921000",
        },
    ),
    PantherRuleTest(
        Name="update",
        ExpectedResult=True,
        Log={
            "protoPayload": {
                "authorizationInfo": [
                    {"granted": True, "permission": "io.k8s.batch.v1.cronjobs.update"}
                ],
                "methodName": "v2.deploymentmanager.deployments.insert",
                "serviceName": "deploymentmanager.googleapis.com",
            },
            "receiveTimestamp": "2024-01-19 13:47:19.465856238",
            "resource": {
                "labels": {"name": "test-vm-deployment", "project_id": "panther-threat-research"},
                "type": "deployment",
            },
            "severity": "NOTICE",
            "timestamp": "2024-01-19 13:47:18.279921000",
        },
    ),
    PantherRuleTest(
        Name="fail",
        ExpectedResult=False,
        Log={
            "protoPayload": {
                "authorizationInfo": [
                    {"granted": False, "permission": "cloudfunctions.functions.upsert"}
                ],
                "methodName": "v2.deploymentmanager.deployments.insert",
                "serviceName": "deploymentmanager.googleapis.com",
            },
            "receiveTimestamp": "2024-01-19 13:47:19.465856238",
            "resource": {
                "labels": {"name": "test-vm-deployment", "project_id": "panther-threat-research"},
                "type": "deployment",
            },
            "severity": "NOTICE",
            "timestamp": "2024-01-19 13:47:18.279921000",
        },
    ),
]


class GCPGKEKubernetesCronJobCreatedOrModified(PantherRule):
    RuleID = "GCP.GKE.Kubernetes.Cron.Job.Created.Or.Modified-prototype"
    DisplayName = "GCP GKE Kubernetes Cron Job Created Or Modified"
    Description = "This detection monitor for any modifications or creations of a cron job in GKE. Attackers may create or modify an existing scheduled job in order to achieve cluster persistence."
    LogTypes = [PantherLogType.GCP_AuditLog]
    Severity = PantherSeverity.Medium
    Reference = "https://medium.com/snowflake/from-logs-to-detection-using-snowflake-and-panther-to-detect-k8s-threats-d72f70a504d7"
    Runbook = "Investigate a reason of creating or modifying a cron job in GKE. Create ticket if appropriate."
    Reports = {"MITRE ATT&CK": ["T1053.003"]}
    Tests = gcpgke_kubernetes_cron_job_created_or_modified_tests

    def rule(self, event):
        authorization_info = deep_walk(event, "protoPayload", "authorizationInfo")
        if not authorization_info:
            return False
        for auth in authorization_info:
            if (
                auth.get("permission")
                in ["io.k8s.batch.v1.cronjobs.create", "io.k8s.batch.v1.cronjobs.update"]
                and auth.get("granted") is True
            ):
                return True
        return False

    def title(self, event):
        actor = deep_get(
            event,
            "protoPayload",
            "authenticationInfo",
            "principalEmail",
            default="<ACTOR_NOT_FOUND>",
        )
        operation = deep_get(event, "protoPayload", "methodName", default="<OPERATION_NOT_FOUND>")
        project_id = deep_get(
            event, "resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>"
        )
        return f"[GCP]: [{actor}] performed [{operation}] on project [{project_id}]"

    def alert_context(self, event):
        return gcp_alert_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.gcp_base_helpers import gcp_alert_context
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

gcpk8s_ioc_activity_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="triggers",
        ExpectedResult=True,
        Log={"operation": {"producer": "k8s.io"}, "p_enrichment": {"tor_exit_nodes": ["1.1.1.1"]}},
    ),
    PantherRuleTest(
        Name="ignore",
        ExpectedResult=False,
        Log={"operation": {"producer": "chrome"}, "p_enrichment": {"tor_exit_nodes": ["1.1.1.1"]}},
    ),
]


class GCPK8sIOCActivity(PantherRule):
    RuleID = "GCP.K8s.IOC.Activity-prototype"
    DisplayName = "GCP K8s IOCActivity"
    LogTypes = [PantherLogType.GCP_AuditLog]
    Tags = ["GCP", "Optional"]
    Severity = PantherSeverity.Medium
    Description = "This detection monitors for any kubernetes API Request originating from an Indicator of Compromise."
    Reports = {"MITRE ATT&CK": ["T1573.002"]}
    Runbook = "Add IP address the request is originated from to banned addresses."
    Reference = "https://medium.com/snowflake/from-logs-to-detection-using-snowflake-and-panther-to-detect-k8s-threats-d72f70a504d7"
    Tests = gcpk8s_ioc_activity_tests

    def rule(self, event):
        if deep_get(event, "operation", "producer") == "k8s.io" and deep_get(
            event, "p_enrichment", "tor_exit_nodes"
        ):
            return True
        return False

    def title(self, event):
        actor = deep_get(
            event,
            "protoPayload",
            "authenticationInfo",
            "principalEmail",
            default="<ACTOR_NOT_FOUND>",
        )
        operation = deep_get(event, "protoPayload", "methodName", default="<OPERATION_NOT_FOUND>")
        project_id = deep_get(
            event, "resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>"
        )
        return f"[GCP]: [{actor}] performed [{operation}] on project [{project_id}]"

    def alert_context(self, event):
        context = gcp_alert_context(event)
        context["tor_exit_nodes"] = deep_get(event, "p_enrichment", "tor_exit_nodes")
        return context



from typing import List

from pypanther.base import PantherDataModel, PantherDataModelMapping
from pypanther.log_types import PantherLogType


class StandardAWSVPCDns(PantherDataModel):
    DataModelID: str = "Standard.AWS.VPCDns"
    DisplayName: str = "AWS VPC DNS"
    Enabled: bool = True
    LogTypes: List[str] = [PantherLogType.AWS_VPCDns]
    Mappings: List[PantherDataModelMapping] = [
        PantherDataModelMapping(Name="source_ip", Path="srcAddr"),
        PantherDataModelMapping(Name="source_port", Path="srcPort"),
        PantherDataModelMapping(Name="dns_query", Path="query_name"),
    ]



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

g_suite_device_compromise_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Normal Mobile Event",
        ExpectedResult=False,
        Log={
            "id": {"applicationName": "mobile"},
            "actor": {"callerType": "USER", "email": "homer.simpson@example.io"},
            "type": "device_updates",
            "name": "DEVICE_REGISTER_UNREGISTER_EVENT",
            "parameters": {"USER_EMAIL": "homer.simpson@example.io"},
        },
    ),
    PantherRuleTest(
        Name="Suspicious Activity Shows not Compromised",
        ExpectedResult=False,
        Log={
            "id": {"applicationName": "mobile"},
            "actor": {"callerType": "USER", "email": "homer.simpson@example.io"},
            "type": "device_updates",
            "name": "DEVICE_COMPROMISED_EVENT",
            "parameters": {
                "USER_EMAIL": "homer.simpson@example.io",
                "DEVICE_COMPROMISED_STATE": "NOT_COMPROMISED",
            },
        },
    ),
    PantherRuleTest(
        Name="Suspicious Activity Shows Compromised",
        ExpectedResult=True,
        Log={
            "id": {"applicationName": "mobile"},
            "actor": {"callerType": "USER", "email": "homer.simpson@example.io"},
            "type": "device_updates",
            "name": "DEVICE_COMPROMISED_EVENT",
            "parameters": {
                "USER_EMAIL": "homer.simpson@example.io",
                "DEVICE_COMPROMISED_STATE": "COMPROMISED",
            },
        },
    ),
]


class GSuiteDeviceCompromise(PantherRule):
    RuleID = "GSuite.DeviceCompromise-prototype"
    DisplayName = "GSuite User Device Compromised"
    LogTypes = [PantherLogType.GSuite_ActivityEvent]
    Tags = ["GSuite"]
    Severity = PantherSeverity.Medium
    Description = "GSuite reported a user's device has been compromised.\n"
    Reference = "https://support.google.com/a/answer/7562165?hl=en&sjid=864417124752637253-EU"
    Runbook = "Have the user change their passwords and reset the device.\n"
    SummaryAttributes = ["actor:email"]
    Tests = g_suite_device_compromise_tests

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "mobile":
            return False
        if event.get("name") == "DEVICE_COMPROMISED_EVENT":
            return bool(deep_get(event, "parameters", "DEVICE_COMPROMISED_STATE") == "COMPROMISED")
        return False

    def title(self, event):
        return f"User [{deep_get(event, 'parameters', 'USER_EMAIL', default='<UNKNOWN_USER>')}]'s device was compromised"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

g_suite_rule_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Non Triggered Rule",
        ExpectedResult=False,
        Log={
            "id": {"applicationName": "rules"},
            "actor": {"email": "some.user@somedomain.com"},
            "parameters": {"severity": "HIGH", "triggered_actions": None},
        },
    ),
    PantherRuleTest(
        Name="High Severity Rule",
        ExpectedResult=True,
        Log={
            "id": {"applicationName": "rules"},
            "actor": {"email": "some.user@somedomain.com"},
            "parameters": {
                "data_source": "DRIVE",
                "severity": "HIGH",
                "triggered_actions": [{"action_type": "DRIVE_UNFLAG_DOCUMENT"}],
            },
        },
    ),
    PantherRuleTest(
        Name="Medium Severity Rule",
        ExpectedResult=True,
        Log={
            "id": {"applicationName": "rules"},
            "actor": {"email": "some.user@somedomain.com"},
            "parameters": {
                "data_source": "DRIVE",
                "severity": "MEDIUM",
                "triggered_actions": [{"action_type": "DRIVE_UNFLAG_DOCUMENT"}],
            },
        },
    ),
    PantherRuleTest(
        Name="Low Severity Rule",
        ExpectedResult=True,
        Log={
            "id": {"applicationName": "rules"},
            "actor": {"email": "some.user@somedomain.com"},
            "parameters": {
                "severity": "LOW",
                "triggered_actions": [{"action_type": "DRIVE_UNFLAG_DOCUMENT"}],
            },
        },
    ),
    PantherRuleTest(
        Name="High Severity Rule with Rule Name",
        ExpectedResult=True,
        Log={
            "id": {"applicationName": "rules"},
            "actor": {"email": "some.user@somedomain.com"},
            "parameters": {
                "severity": "HIGH",
                "rule_name": "CEO Impersonation",
                "triggered_actions": [{"action_type": "MAIL_MARK_AS_PHISHING"}],
            },
        },
    ),
]


class GSuiteRule(PantherRule):
    RuleID = "GSuite.Rule-prototype"
    DisplayName = "GSuite Passthrough Rule Triggered"
    LogTypes = [PantherLogType.GSuite_ActivityEvent]
    Tags = ["GSuite"]
    Severity = PantherSeverity.Info
    Description = "A GSuite rule was triggered.\n"
    Reference = "https://support.google.com/a/answer/9420866"
    Runbook = "Investigate what triggered the rule.\n"
    SummaryAttributes = ["actor:email"]
    Tests = g_suite_rule_tests

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "rules":
            return False
        if not deep_get(event, "parameters", "triggered_actions"):
            return False
        return True

    def title(self, event):
        rule_severity = deep_get(event, "parameters", "severity")
        if deep_get(event, "parameters", "rule_name"):
            return (
                "GSuite "
                + rule_severity
                + " Severity Rule Triggered: "
                + deep_get(event, "parameters", "rule_name")
            )
        return "GSuite " + rule_severity + " Severity Rule Triggered"

    def severity(self, event):
        return deep_get(event, "parameters", "severity", default="INFO")



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import slack_alert_context
from pypanther.log_types import PantherLogType

slack_audit_logs_information_barrier_modified_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Information Barrier Deleted",
        ExpectedResult=True,
        Log={
            "action": "barrier_deleted",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "A012B3CDEFG",
                    "name": "username",
                    "team": "T01234N56GB",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace",
                    "id": "T01234N56GB",
                    "name": "test-workspace",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
        },
    ),
    PantherRuleTest(
        Name="Information Barrier Updated",
        ExpectedResult=True,
        Log={
            "action": "barrier_updated",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "A012B3CDEFG",
                    "name": "username",
                    "team": "T01234N56GB",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace",
                    "id": "T01234N56GB",
                    "name": "test-workspace",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
        },
    ),
    PantherRuleTest(
        Name="User Logout",
        ExpectedResult=False,
        Log={
            "action": "user_logout",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "T01234N56GB",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace-1",
                    "id": "T01234N56GB",
                    "name": "test-workspace-1",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
            "date_create": "2022-07-28 15:22:32",
            "entity": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "T01234N56GB",
                },
            },
            "id": "72cac009-9eb3-4dde-bac6-ee49a32a1789",
        },
    ),
]


class SlackAuditLogsInformationBarrierModified(PantherRule):
    RuleID = "Slack.AuditLogs.InformationBarrierModified-prototype"
    DisplayName = "Slack Information Barrier Modified"
    LogTypes = [PantherLogType.Slack_AuditLogs]
    Tags = ["Slack", "Defense Evasion", "Impair Defenses", "Disable or Modify Tools"]
    Reports = {"MITRE ATT&CK": ["TA0005:T1562.001"]}
    Severity = PantherSeverity.Medium
    Description = "Detects when a Slack information barrier is deleted/updated"
    Reference = "https://slack.com/intl/en-gb/help/articles/360056171734-Create-information-barriers-in-Slack"
    SummaryAttributes = ["action", "p_any_ip_addresses", "p_any_emails"]
    Tests = slack_audit_logs_information_barrier_modified_tests
    INFORMATION_BARRIER_ACTIONS = {
        "barrier_deleted": "Slack Information Barrier Deleted",
        "barrier_updated": "Slack Information Barrier Updated",
    }

    def rule(self, event):
        return event.get("action") in self.INFORMATION_BARRIER_ACTIONS

    def title(self, event):
        if event.get("action") in self.INFORMATION_BARRIER_ACTIONS:
            return self.INFORMATION_BARRIER_ACTIONS.get(event.get("action"))
        return "Slack Information Barrier Modified"

    def alert_context(self, event):
        return slack_alert_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import pattern_match_list
from pypanther.log_types import PantherLogType

teleport_create_user_accounts_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Echo command",
        ExpectedResult=False,
        Log={
            "argv": [],
            "cgroup_id": 4294967537,
            "code": "T4000I",
            "ei": 15,
            "event": "session.command",
            "login": "root",
            "namespace": "default",
            "path": "/bin/echo",
            "pid": 7143,
            "ppid": 7115,
            "program": "echo",
            "return_code": 0,
            "server_id": "e75992b4-9e27-456f-b1c9-7a32da83c661",
            "sid": "8a3fc038-785b-43f3-8737-827b3e25fe5b",
            "time": "2020-08-17T17:40:37.491Z",
            "uid": "8eaf8f39-09d4-4a42-a22a-65163d2af702",
            "user": "panther",
        },
    ),
    PantherRuleTest(
        Name="Userdel command",
        ExpectedResult=True,
        Log={
            "argv": ["jacknew"],
            "cgroup_id": 4294967567,
            "code": "T4000I",
            "ei": 105,
            "event": "session.command",
            "login": "root",
            "namespace": "default",
            "path": "/sbin/userdel",
            "pid": 8931,
            "ppid": 8930,
            "program": "userdel",
            "return_code": 0,
            "server_id": "e75992b4-9e27-456f-b1c9-7a32da83c661",
            "sid": "4244c271-8069-4679-a27e-f7c18f88ce45",
            "time": "2020-08-17T18:39:26.192Z",
            "uid": "346d3f61-a010-4871-84de-897f50b18118",
            "user": "panther",
        },
    ),
]


class TeleportCreateUserAccounts(PantherRule):
    RuleID = "Teleport.CreateUserAccounts-prototype"
    DisplayName = "Teleport Create User Accounts"
    LogTypes = [PantherLogType.Gravitational_TeleportAudit]
    Tags = ["SSH", "Persistence:Create Account"]
    Reports = {"MITRE ATT&CK": ["TA0003:T1136"]}
    Severity = PantherSeverity.High
    Description = "A user has been manually created, modified, or deleted"
    DedupPeriodMinutes = 15
    Reference = "https://goteleport.com/docs/management/admin/"
    Runbook = "Analyze why it was manually created and delete it if necessary."
    SummaryAttributes = [
        "event",
        "code",
        "user",
        "program",
        "path",
        "return_code",
        "login",
        "server_id",
        "sid",
    ]
    Tests = teleport_create_user_accounts_tests  # user password expiry
    # change passwords for users
    # create, modify, and delete users
    USER_CREATE_PATTERNS = ["chage", "passwd", "user*"]

    def rule(self, event):
        # Filter the events
        if event.get("event") != "session.command":
            return False
        # Check that the program matches our list above
        return pattern_match_list(event.get("program", ""), self.USER_CREATE_PATTERNS)

    def title(self, event):
        return f"User [{event.get('user', '<UNKNOWN_USER>')}] has manually modified system users on [{event.get('cluster_name', '<UNKNOWN_CLUSTER>')}]"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_notion_helpers import notion_alert_context
from pypanther.log_types import PantherLogType


class NotionPageSharedToWeb(PantherRule):
    RuleID = "Notion.PageSharedToWeb-prototype"
    DisplayName = "Notion Page Published to Web"
    LogTypes = [PantherLogType.Notion_AuditLogs]
    Tags = ["Notion", "Data Security", "Information Disclosure"]
    Severity = PantherSeverity.Low
    Description = "A Notion User published a page to the web."
    Runbook = "Potential information exposure - review the shared page and rectify if needed."
    Reference = "https://www.notion.so/help/public-pages-and-web-publishing"
    # These event types correspond to users adding or editing the default role on a public page
    event_types = (
        "page.permissions.shared_to_public_role_added",
        "page.permissions.shared_to_public_role_updated",
    )

    def rule(self, event):
        return event.deep_get("event", "type", default="<NO_EVENT_TYPE_FOUND>") in self.event_types

    def title(self, event):
        user = event.deep_get("event", "actor", "person", "email", default="<NO_USER_FOUND>")
        page_name = event.deep_get("event", "details", "page_name", default="<NO_PAGE_NAME_FOUND>")
        return f"Notion User [{user}] changed the status of page [{page_name}] to public."

    def alert_context(self, event):
        context = notion_alert_context(event)
        page_name = event.deep_get("event", "details", "page_name", default="<NO_PAGE_NAME_FOUND>")
        context["page_name"] = page_name
        return context



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_snyk_helpers import snyk_alert_context
from pypanther.log_types import PantherLogType

snyk_project_settings_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Snyk Org Project Stop Monitor",
        ExpectedResult=True,
        Log={
            "content": {
                "origin": "github",
                "target": {
                    "branch": "some-branch",
                    "id": 222222222,
                    "name": "repo-name",
                    "owner": "github-org",
                },
                "targetFile": "go.mod",
                "type": "gomodules",
            },
            "created": "2023-03-30 15:38:18.58",
            "event": "org.project.stop_monitor",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "orgId": "21111111-a222-4eee-8ddd-a99999999999",
            "projectId": "05555555-8555-2333-5aaa-600000000000",
            "userId": "05555555-3333-4ddd-8ccc-75555555555",
        },
    ),
    PantherRuleTest(
        Name="Project Ignore Create",
        ExpectedResult=True,
        Log={
            "content": {
                "created": "2023-03-20T12:23:06.356Z",
                "ignorePath": "*",
                "ignoredBy": {"id": "05555555-3333-4ddd-8ccc-75555555555"},
                "issueId": "SNYK-JS-UNDICI-3323845",
                "reason": "dev dependency",
                "reasonType": "wont-fix",
            },
            "created": "2023-03-20 12:23:08.363",
            "event": "org.project.ignore.create",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
            "orgId": "21111111-a222-4eee-8ddd-a99999999999",
            "projectId": "05555555-8555-2333-5aaa-600000000000",
            "userId": "05555555-3333-4ddd-8ccc-75555555555",
        },
    ),
    PantherRuleTest(
        Name="Snyk Group SSO Membership sync",
        ExpectedResult=False,
        Log={
            "content": {},
            "created": "2023-03-15 13:13:13.133",
            "event": "group.sso.membership.sync",
            "groupId": "8fffffff-1555-4444-b000-b55555555555",
        },
    ),
]


class SnykProjectSettings(PantherRule):
    RuleID = "Snyk.Project.Settings-prototype"
    DisplayName = "Snyk Project Settings"
    LogTypes = [PantherLogType.Snyk_GroupAudit, PantherLogType.Snyk_OrgAudit]
    Tags = ["Snyk"]
    Reference = "https://docs.snyk.io/snyk-admin/introduction-to-snyk-projects/view-and-edit-project-settings"
    Severity = PantherSeverity.Medium
    Description = "Detects when Snyk Project settings are changed\n"
    SummaryAttributes = ["event"]
    Tests = snyk_project_settings_tests
    # The bodies of these actions are quite diverse.
    # When projects are added, the logged detail is the sourceOrgId.
    # org.project.stop_monitor is logged for individual files
    #   that are ignored.
    # AND the equivalent for licenses",
    ACTIONS = [
        "org.sast_settings.edit",
        "org.project.attributes.edit",
        "org.project.add",
        "org.project.delete",
        "org.project.edit",
        "org.project.fix_pr.manual_open",
        "org.project.ignore.create",
        "org.project.ignore.delete",
        "org.project.ignore.edit",
        "org.project.monitor",
        "org.project.pr_check.edit",
        "org.project.remove",
        "org.project.settings.delete",
        "org.project.settings.edit",
        "org.project.stop_monitor",
        "org.license_rule.create",
        "org.license_rule.delete",
        "org.license_rule.edit",
    ]

    def rule(self, event):
        action = deep_get(event, "event", default="<NO_EVENT>")
        return action in self.ACTIONS

    def title(self, event):
        group_or_org = "<GROUP_OR_ORG>"
        operation = "<NO_OPERATION>"
        action = deep_get(event, "event", default="<NO_EVENT>")
        if "." in action:
            group_or_org = action.split(".")[0].title()
            operation = ".".join(action.split(".")[1:]).title()
        return f"Snyk: [{group_or_org}] [{operation}] performed by [{deep_get(event, 'userId', default='<NO_USERID>')}]"

    def alert_context(self, event):
        a_c = snyk_alert_context(event)
        # merge event in for the alert_context
        a_c.update(event)
        return a_c

    def dedup(self, event):
        return f"{deep_get(event, 'userId', default='<NO_USERID>')}{deep_get(event, 'orgId', default='<NO_ORGID>')}{deep_get(event, 'groupId', default='<NO_GROUPID>')}{deep_get(event, 'event', default='<NO_EVENT>')}"

    def severity(self, event):
        action = deep_get(event, "event", default="<NO_EVENT>")
        if action == "org.project.fix_pr.manual_open":
            return "INFO"
        return "LOW"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

box_new_login_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Regular Event",
        ExpectedResult=False,
        Log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {
                "id": "12345678",
                "type": "user",
                "login": "cat@example",
                "name": "Bob Cat",
            },
            "event_type": "DELETE",
        },
    ),
    PantherRuleTest(
        Name="New Login Event",
        ExpectedResult=True,
        Log={
            "type": "event",
            "additional_details": '{"key": "value"}',
            "created_by": {
                "id": "12345678",
                "type": "user",
                "login": "cat@example",
                "name": "Bob Cat",
            },
            "event_type": "ADD_LOGIN_ACTIVITY_DEVICE",
            "source": {"id": "12345678", "type": "user", "login": "user@example"},
        },
    ),
]


class BoxNewLogin(PantherRule):
    RuleID = "Box.New.Login-prototype"
    DisplayName = "Box New Login"
    LogTypes = [PantherLogType.Box_Event]
    Tags = ["Box", "Initial Access:Valid Accounts"]
    Reports = {"MITRE ATT&CK": ["TA0001:T1078"]}
    Severity = PantherSeverity.Info
    Description = "A user logged in from a new device.\n"
    Reference = "https://support.box.com/hc/en-us/articles/360043691914-Controlling-Devices-Used-to-Access-Box"
    Runbook = "Investigate whether this is a valid user login.\n"
    SummaryAttributes = ["ip_address"]
    Tests = box_new_login_tests

    def rule(self, event):
        # ADD_LOGIN_ACTIVITY_DEVICE
        #  detect when a user logs in from a device not previously seen
        return event.get("event_type") == "ADD_LOGIN_ACTIVITY_DEVICE"

    def title(self, event):
        return f"User [{deep_get(event, 'created_by', 'name', default='<UNKNOWN_USER>')}] logged in from a new device."



from typing import List

from pypanther.base import PantherDataModel, PantherDataModelMapping
from pypanther.log_types import PantherLogType


class StandardCloudflareHttpReq(PantherDataModel):
    DataModelID: str = "Standard.Cloudflare.HttpReq"
    DisplayName: str = "Cloudflare Firewall"
    Enabled: bool = True
    LogTypes: List[str] = [PantherLogType.Cloudflare_HttpRequest]
    Mappings: List[PantherDataModelMapping] = [
        PantherDataModelMapping(Name="source_ip", Path="ClientIP"),
        PantherDataModelMapping(Name="user_agent", Path="ClientRequestUserAgent"),
        PantherDataModelMapping(Name="http_status", Path="EdgeResponseStatus"),
        PantherDataModelMapping(Name="source_port", Path="ClientSrcPort"),
    ]



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

decoy_systems_manager_parameter_accessed_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Systems-Manager-Parameter-Decoy-Accessed",
        ExpectedResult=True,
        Log={
            "Action": {
                "ActionType": "AWS_API_CALL",
                "AwsApiCallAction": {
                    "Api": "Decrypt",
                    "CallerType": "remoteIp",
                    "DomainDetails": {},
                    "ServiceName": "kms.amazonaws.com",
                },
                "DnsRequestAction": {},
                "NetworkConnectionAction": {"LocalPortDetails": {}, "RemotePortDetails": {}},
                "PortProbeAction": {},
            },
            "AwsAccountId": "123456789012",
            "CompanyName": "Custom",
            "CreatedAt": "2024-05-24 22:34:07.000000000",
            "Description": "Private decoy Systems Manager parameter arn:aws:ssm:us-east-1:123456789012:parameter/info-parameter was accessed by arn:aws:iam::123456789012:user/tester. This Systems Manager parameter has been provisioned to monitor and generate security events when accessed and can be an indicator of unintended or unauthorized access to your AWS Account.",
            "FindingProviderFields": {
                "Severity": {"Label": "HIGH", "Normalized": 70},
                "Types": ["Unusual Behaviors"],
            },
            "GeneratorId": "ssm.amazonaws.com",
            "Id": "6abc0de0-69ea-4e15-91c6-27eb4a07bd21",
            "ProcessedAt": "2024-05-24T22:34:15.644Z",
            "ProductArn": "arn:aws:securityhub:us-east-1:123456789012:product/123456789012/default",
            "ProductFields": {
                "Custom/DecoyDetector/apiResult": "SUCCESS",
                "Custom/DecoyDetector/requestID": "ab8cd646-1986-4c45-8546-fdb1776e23b0",
                "Custom/DecoyDetector/userAgent": "AWS Internal",
                "aws/securityhub/CompanyName": "Personal",
                "aws/securityhub/FindingId": "arn:aws:service:region:123456789012:resource/12345ab6-436d-4d59-ac58-ed6b3127e440",
                "aws/securityhub/ProductName": "Default",
            },
            "ProductName": "DecoyDetector",
            "RecordState": "ACTIVE",
            "Region": "us-east-1",
            "Resources": [
                {
                    "Id": "arn:aws:ssm:us-east-1:123456789012:parameter/info-parameter",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Target",
                    "Type": "Other",
                },
                {
                    "Id": "arn:aws:kms:us-east-1:123456789012:key/007ab31c-bf66-2264-a916-49f6d2ebd1db",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Target",
                    "Type": "AwsKmsKey",
                },
                {
                    "Details": {
                        "AwsIamAccessKey": {
                            "AccessKeyId": "ABC9ONWNS3155VIEJC8U",
                            "AccountId": "123456789012",
                            "PrincipalId": "ABCDEFG0TOGJSGNQKI0:john.doe",
                            "PrincipalType": "AssumedRole",
                            "SessionContext": {
                                "Attributes": {
                                    "CreationDate": "2024-05-24T22:32:38Z",
                                    "MfaAuthenticated": False,
                                },
                                "SessionIssuer": {
                                    "AccountId": "123456789012",
                                    "Arn": "arn:aws:iam::123456789012:user/tester",
                                    "PrincipalId": "ABCDEFG0TOGJSGNQKI0",
                                    "Type": "Role",
                                    "UserName": "user_ab21cde50f",
                                },
                            },
                        }
                    },
                    "Id": "ABC9ONWNS3155VIEJC8U",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Actor",
                    "Type": "AwsIamAccessKey",
                },
                {
                    "Id": "arn:aws:iam::123456789012:user/tester",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Actor",
                    "Type": "AwsIamRole",
                },
            ],
            "SchemaVersion": "2018-10-08",
            "Severity": {"Label": "HIGH", "Normalized": 70},
            "Title": "Suspicious activity detected accessing private decoy Systems Manager parameter arn:aws:ssm:us-east-1:123456789012:parameter/info-parameter",
            "Types": ["Unusual Behaviors"],
            "UpdatedAt": "2024-05-24 22:34:07.000000000",
            "Workflow": {"Status": "NEW"},
            "WorkflowState": "NEW",
            "p_any_actor_ids": [],
            "p_any_aws_account_ids": [],
            "p_any_aws_arns": [],
            "p_any_trace_ids": [],
            "p_any_usernames": [],
            "p_event_time": "2024-05-24 22:34:07.000000000",
            "p_log_type": "AWS.SecurityFindingFormat",
            "p_parse_time": "2024-05-24 22:35:04.272574202",
            "p_row_id": "zjj8nmnw9f90uulxfa3bmen8rv5stlcx",
            "p_schema_version": 0,
            "p_source_id": "bb4e16c5-43dd-450c-9227-39f0d152659c",
            "p_source_label": "AWS Security Hub",
            "p_udm": {},
        },
    ),
    PantherRuleTest(
        Name="Systems-Manager-Parameter-Decoy-Not-Accessed",
        ExpectedResult=False,
        Log={
            "Action": {
                "ActionType": "AWS_API_CALL",
                "AwsApiCallAction": {
                    "Api": "DescribeParameters",
                    "CallerType": "remoteIp",
                    "DomainDetails": {},
                    "ServiceName": "kms.amazonaws.com",
                },
                "DnsRequestAction": {},
                "NetworkConnectionAction": {"LocalPortDetails": {}, "RemotePortDetails": {}},
                "PortProbeAction": {},
            },
            "AwsAccountId": "123456789012",
            "CompanyName": "Custom",
            "CreatedAt": "2024-05-24 22:34:07.000000000",
            "Description": "Private decoy Systems Manager parameter arn:aws:ssm:us-east-1:123456789012:parameter/info-parameter was not accessed by arn:aws:iam::123456789012:user/tester. This Systems Manager parameter has been provisioned to monitor and generate security events when accessed and can be an indicator of unintended or unauthorized access to your AWS Account.",
            "FindingProviderFields": {
                "Severity": {"Label": "HIGH", "Normalized": 70},
                "Types": ["Unusual Behaviors"],
            },
            "GeneratorId": "ssm.amazonaws.com",
            "Id": "6abc0de0-69ea-4e15-91c6-27eb4a07bd21",
            "ProcessedAt": "2024-05-24T22:34:15.644Z",
            "ProductArn": "arn:aws:securityhub:us-east-1:123456789012:product/123456789012/default",
            "ProductFields": {
                "Custom/DecoyDetector/apiResult": "SUCCESS",
                "Custom/DecoyDetector/requestID": "ab1cd234-1986-4c45-8546-fdb1776e23b0",
                "Custom/DecoyDetector/userAgent": "AWS Internal",
                "aws/securityhub/CompanyName": "Personal",
                "aws/securityhub/FindingId": "arn:aws:service:region:123456789012:resource/12345ab6-436d-4d59-ac58-ed6b3127e440",
                "aws/securityhub/ProductName": "Default",
            },
            "ProductName": "DecoyDetector",
            "RecordState": "ACTIVE",
            "Region": "us-east-1",
            "Resources": [
                {
                    "Id": "arn:aws:ssm:us-east-1:123456789012:parameter/info-parameter",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Target",
                    "Type": "Other",
                },
                {
                    "Id": "arn:aws:kms:us-east-1:123456789012:key/007ab31c-bf66-2264-a916-49f6d2ebd1db",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Target",
                    "Type": "AwsKmsKey",
                },
                {
                    "Details": {
                        "AwsIamAccessKey": {
                            "AccessKeyId": "ABC9ONWNS3155VIEJC8U",
                            "AccountId": "123456789012",
                            "PrincipalId": "ABCDEFG0TOGJSGNQKI0:john.doe",
                            "PrincipalType": "AssumedRole",
                            "SessionContext": {
                                "Attributes": {
                                    "CreationDate": "2024-05-24T22:32:38Z",
                                    "MfaAuthenticated": False,
                                },
                                "SessionIssuer": {
                                    "AccountId": "123456789012",
                                    "Arn": "arn:aws:iam::123456789012:user/tester",
                                    "PrincipalId": "ABCDEFG0TOGJSGNQKI0",
                                    "Type": "Role",
                                    "UserName": "user_ab21cde50f",
                                },
                            },
                        }
                    },
                    "Id": "ABC9ONWNS3155VIEJC8U",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Actor",
                    "Type": "AwsIamAccessKey",
                },
                {
                    "Id": "arn:aws:iam::123456789012:user/tester",
                    "Partition": "aws",
                    "Region": "us-east-1",
                    "ResourceRole": "Actor",
                    "Type": "AwsIamRole",
                },
            ],
            "SchemaVersion": "2018-10-08",
            "Severity": {"Label": "HIGH", "Normalized": 70},
            "Title": "Non-Suspicious activity detected accessing private decoy Systems Manager parameter arn:aws:ssm:us-east-1:123456789012:parameter/info-parameter",
            "Types": ["Unusual Behaviors"],
            "UpdatedAt": "2024-05-24 22:34:07.000000000",
            "Workflow": {"Status": "NEW"},
            "WorkflowState": "NEW",
            "p_any_actor_ids": [],
            "p_any_aws_account_ids": [],
            "p_any_aws_arns": [],
            "p_any_trace_ids": [],
            "p_any_usernames": [],
            "p_event_time": "2024-05-24 22:34:07.000000000",
            "p_log_type": "AWS.SecurityFindingFormat",
            "p_parse_time": "2024-05-24 22:35:04.272574202",
            "p_row_id": "zjj8nmnw9f90uulxfa3bmen8rv5stlcx",
            "p_schema_version": 0,
            "p_source_id": "bb4e16c5-43dd-450c-9227-39f0d152659c",
            "p_source_label": "AWS Security Hub",
            "p_udm": {},
        },
    ),
]


class DecoySystemsManagerParameterAccessed(PantherRule):
    RuleID = "Decoy.Systems.Manager.Parameter.Accessed-prototype"
    DisplayName = "Decoy Systems Manager Parameter Accessed"
    Enabled = False
    LogTypes = [PantherLogType.AWS_SecurityFindingFormat]
    Severity = PantherSeverity.High
    Description = "Actor accessed Decoy Systems Manager parameter"
    Reference = "https://aws.amazon.com/blogs/security/how-to-detect-suspicious-activity-in-your-aws-account-by-using-private-decoy-resources/"
    InlineFilters = [{"All": []}]
    Tests = decoy_systems_manager_parameter_accessed_tests

    def rule(self, event):
        # List of suspicious API events
        # NOTE: There may be more API events that's not listed
        suspicious_api_events = ["Decrypt"]
        # Return True if the API value is in the list of suspicious API events
        if event["GeneratorId"] == "ssm.amazonaws.com":
            # Extract the API value from the event
            api_value = event["Action"]["AwsApiCallAction"]["Api"]
            return api_value in suspicious_api_events
        return False

    def title(self, event):
        # (Optional) Return a string which will be shown as the alert title.
        # If no 'dedup' function is defined, the return value of this method will act as dedup string.
        # NOTE: Not sure if the offending actor Id will always be in the 0th index of Resources
        # It's possible to just return the Title as a whole string
        secret = event["Resources"][0]["Id"]
        return f"Suspicious activity detected accessing     private decoy Systems Manager parameter {secret}"



import re
from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import ZENDESK_CHANGE_DESCRIPTION
from pypanther.log_types import PantherLogType

zendesk_account_owner_changed_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Zendesk - Owner Changed",
        ExpectedResult=True,
        Log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "account",
            "source_label": "Account: Account",
            "action": "update",
            "change_description": "Owner changed from Bob Cat to Mountain Lion",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
    PantherRuleTest(
        Name="Zendesk - Admin Role Assigned",
        ExpectedResult=False,
        Log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "user",
            "source_label": "Account: Account",
            "action": "update",
            "change_description": "Role changed from End User to Administrator",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
]


class ZendeskAccountOwnerChanged(PantherRule):
    RuleID = "Zendesk.AccountOwnerChanged-prototype"
    DisplayName = "Zendesk Account Owner Changed"
    LogTypes = [PantherLogType.Zendesk_Audit]
    Severity = PantherSeverity.High
    Tags = ["Zendesk", "Privilege Escalation:Valid Accounts"]
    Reports = {"MITRE ATT&CK": ["TA0004:T1078"]}
    Description = (
        "Only one admin user can be the account owner. Ensure the change in ownership is expected."
    )
    Reference = (
        "https://support.zendesk.com/hc/en-us/articles/4408822084634-Changing-the-account-owner"
    )
    SummaryAttributes = ["p_any_ip_addresses"]
    Tests = zendesk_account_owner_changed_tests
    ZENDESK_OWNER_CHANGED = re.compile(
        "Owner changed from (?P<old_owner>.+) to (?P<new_owner>[^$]+)", re.IGNORECASE
    )

    def rule(self, event):
        if event.get("action", "") == "update" and event.get("source_type", "") == "account":
            return (
                event.get(ZENDESK_CHANGE_DESCRIPTION, "").lower().startswith("owner changed from ")
            )
        return False

    def title(self, event):
        old_owner = "<UNKNOWN_USER>"
        new_owner = "<UNKNOWN_USER>"
        matches = self.ZENDESK_OWNER_CHANGED.match(event.get(ZENDESK_CHANGE_DESCRIPTION, ""))
        if matches:
            old_owner = matches.group("old_owner")
            new_owner = matches.group("new_owner")
        return f"zendesk administrative owner changed from {old_owner} to {new_owner}"



from typing import List

import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import PantherDataModel, PantherDataModelMapping
from pypanther.log_types import PantherLogType


def get_event_type(event):
    if event.get("type") == "Sign in":
        return event_type.SUCCESSFUL_LOGIN
    if event.get("type") == "Sign out":
        return event_type.SUCCESSFUL_LOGOUT
    return None


class StandardZoomActivity(PantherDataModel):
    DataModelID: str = "Standard.Zoom.Activity"
    DisplayName: str = "Zoom Activity"
    Enabled: bool = True
    LogTypes: List[str] = [PantherLogType.Zoom_Activity]
    Mappings: List[PantherDataModelMapping] = [
        PantherDataModelMapping(Name="actor_user", Path="email"),
        PantherDataModelMapping(Name="event_type", Method=get_event_type),
        PantherDataModelMapping(Name="source_ip", Path="ip_address"),
    ]



from importlib import import_module
from pkgutil import walk_packages
from typing import List, Set, Type

from prettytable import PrettyTable

from pypanther.base import PantherDataModel, PantherRule

__RULES: Set[Type[PantherRule]] = set()


def __to_set(value):
    if isinstance(value, str):
        return {value}
    try:
        return set(value)
    except TypeError:
        return {value}


def get_panther_rules(**kwargs):
    """Return an iterator of all PantherRules in the pypanther.rules based on the provided filters.
    If the filter argument is not provided, all rules are returned. If a filter value is a list, any value in the
    list will match. If a filter value is a string, the value must match exactly.
    """
    if not __RULES:
        p_a_r = import_module("pypanther.rules")
        for module_info in walk_packages(p_a_r.__path__, "pypanther.rules."):
            if len(module_info.name.split(".")) > 3:
                m = import_module(module_info.name)
                for item in dir(m):
                    attr = getattr(m, item)
                    if (
                        isinstance(attr, type)
                        and issubclass(attr, PantherRule)
                        and attr is not PantherRule
                    ):
                        if not hasattr(attr, "RuleID"):
                            continue
                        __RULES.add(attr)

    return filter_kwargs(__RULES, **kwargs)


__DATA_MODELS: Set[Type[PantherRule]] = set()


def get_panther_data_models(**kwargs):
    """Return an iterator of all PantherDataModels in the pypanther.rules based on the provided filters.
    If the filter argument is not provided, all data models are returned. If a filter value is a list, any value in the
    list will match. If a filter value is a string, the value must match exactly.
    """
    if not __DATA_MODELS:
        p_a_d = import_module("pypanther.data_models")
        for module_info in walk_packages(p_a_d.__path__, "pypanther.data_models."):
            m = import_module(module_info.name)
            for item in dir(m):
                attr = getattr(m, item)
                if (
                    isinstance(attr, type)
                    and issubclass(attr, PantherDataModel)
                    and attr is not PantherDataModel
                ):
                    __DATA_MODELS.add(attr)

    return filter_kwargs(__DATA_MODELS, **kwargs)


# Get rules based on filter criteria
def filter_kwargs(
    iterable,
    **kwargs,
):
    return [
        x
        for x in iterable
        if all(
            __to_set(getattr(x, key, set())).intersection(__to_set(values))
            for key, values in kwargs.items()
        )
    ]


# Prints rules in a table format for easy viewing
def table_print(rules: List[PantherRule]) -> PrettyTable:
    table = PrettyTable()
    table.field_names = [
        "RuleID",
        "LogTypes",
        "DisplayName",
        "Severity",
        "Enabled",
        "CreateAlert",
    ]
    for rule in rules:
        log_types = rule.LogTypes
        if len(log_types) > 2:
            log_types = log_types[:2] + ["+{}".format(len(log_types) - 2)]

        table.add_row(
            [
                rule.RuleID,
                ", ".join([str(s) for s in log_types]),
                rule.DisplayName,
                rule.Severity,
                rule.Enabled,
                rule.CreateAlert,
            ]
        )
    table.sortby = "RuleID"
    # table.reversesort = True
    return table



from fnmatch import fnmatch
from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get
from pypanther.helpers.panther_default import aws_cloudtrail_success
from pypanther.log_types import PantherLogType

aws_cloud_trail_security_configuration_change_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Security Configuration Changed",
        ExpectedResult=True,
        Log={
            "awsRegion": "us-west-2",
            "eventID": "1111",
            "eventName": "DeleteTrail",
            "eventSource": "cloudtrail.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "readOnly": False,
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {"name": "example-trail"},
            "responseElements": None,
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "invokedBy": "cloudformation.amazonaws.com",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {
                        "creationDate": "2019-01-01T00:00:00Z",
                        "mfaAuthenticated": "true",
                    },
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    PantherRuleTest(
        Name="Security Configuration Not Changed",
        ExpectedResult=False,
        Log={
            "awsRegion": "us-west-2",
            "eventID": "1111",
            "eventName": "DescribeTrail",
            "eventSource": "cloudtrail.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "readOnly": False,
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {"name": "example-trail"},
            "responseElements": None,
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "invokedBy": "cloudformation.amazonaws.com",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {
                        "creationDate": "2019-01-01T00:00:00Z",
                        "mfaAuthenticated": "true",
                    },
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    PantherRuleTest(
        Name="Non Security Configuration Change",
        ExpectedResult=False,
        Log={
            "awsRegion": "us-west-2",
            "eventID": "1111",
            "eventName": "PutPolicy",
            "eventSource": "iam.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "readOnly": False,
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {"name": "example-trail"},
            "responseElements": None,
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "invokedBy": "cloudformation.amazonaws.com",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {
                        "creationDate": "2019-01-01T00:00:00Z",
                        "mfaAuthenticated": "true",
                    },
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    PantherRuleTest(
        Name="Security Configuration Not Changed - Error",
        ExpectedResult=False,
        Log={
            "awsRegion": "us-west-2",
            "errorCode": "ConflictException",
            "eventID": "1111",
            "eventName": "DeleteTrail",
            "eventSource": "cloudtrail.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "readOnly": False,
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {"name": "example-trail"},
            "responseElements": None,
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "invokedBy": "cloudformation.amazonaws.com",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {
                        "creationDate": "2019-01-01T00:00:00Z",
                        "mfaAuthenticated": "true",
                    },
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    PantherRuleTest(
        Name="Security Configuration Changed - Allowlisted User",
        ExpectedResult=False,
        Log={
            "awsRegion": "us-west-2",
            "eventID": "1111",
            "eventName": "ExampleEvent",
            "eventSource": "cloudtrail.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "readOnly": False,
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {"name": "example-trail"},
            "responseElements": None,
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "invokedBy": "cloudformation.amazonaws.com",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {
                        "creationDate": "2019-01-01T00:00:00Z",
                        "mfaAuthenticated": "true",
                    },
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "ExampleUser",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
]


class AWSCloudTrailSecurityConfigurationChange(PantherRule):
    RuleID = "AWS.CloudTrail.SecurityConfigurationChange-prototype"
    DisplayName = "Account Security Configuration Changed"
    LogTypes = [PantherLogType.AWS_CloudTrail]
    Tags = ["AWS", "Defense Evasion:Impair Defenses"]
    Severity = PantherSeverity.Medium
    Reports = {"MITRE ATT&CK": ["TA0005:T1562"]}
    Description = "An account wide security configuration was changed."
    Runbook = "Verify that this change was planned. If not, revert the change and update the access control policies to ensure this doesn't happen again.\n"
    Reference = "https://docs.aws.amazon.com/prescriptive-guidance/latest/aws-startup-security-baseline/controls-acct.html"
    SummaryAttributes = [
        "eventName",
        "userAgent",
        "sourceIpAddress",
        "recipientAccountId",
        "p_any_aws_arns",
    ]
    Tests = aws_cloud_trail_security_configuration_change_tests
    SECURITY_CONFIG_ACTIONS = {
        "DeleteAccountPublicAccessBlock",
        "DeleteDeliveryChannel",
        "DeleteDetector",
        "DeleteFlowLogs",
        "DeleteRule",
        "DeleteTrail",
        "DisableEbsEncryptionByDefault",
        "DisableRule",
        "StopConfigurationRecorder",
        "StopLogging",
    }
    # Add expected events and users here to suppress alerts
    ALLOW_LIST = [{"userName": "ExampleUser", "eventName": "ExampleEvent"}]

    def rule(self, event):
        if not aws_cloudtrail_success(event):
            return False
        for entry in self.ALLOW_LIST:
            if fnmatch(
                deep_get(
                    event, "userIdentity", "sessionContext", "sessionIssuer", "userName", default=""
                ),
                entry["userName"],
            ):
                if fnmatch(event.get("eventName"), entry["eventName"]):
                    return False
        if event.get("eventName") == "UpdateDetector":
            return not deep_get(event, "requestParameters", "enable", default=True)
        return event.get("eventName") in self.SECURITY_CONFIG_ACTIONS

    def title(self, event):
        user = deep_get(event, "userIdentity", "userName") or deep_get(
            event, "userIdentity", "sessionContext", "sessionIssuer", "userName"
        )
        return f"Sensitive AWS API call {event.get('eventName')} made by {user}"

    def alert_context(self, event):
        return aws_rule_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.log_types import PantherLogType

g_suite_device_suspicious_activity_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Normal Mobile Event",
        ExpectedResult=False,
        Log={
            "id": {"applicationName": "mobile"},
            "actor": {"callerType": "USER", "email": "homer.simpson@example.io"},
            "type": "device_updates",
            "name": "DEVICE_SYNC_EVENT",
            "parameters": {"USER_EMAIL": "homer.simpson@example.io"},
        },
    ),
    PantherRuleTest(
        Name="Suspicious Activity",
        ExpectedResult=True,
        Log={
            "id": {"applicationName": "mobile"},
            "actor": {"callerType": "USER", "email": "homer.simpson@example.io"},
            "type": "device_updates",
            "name": "SUSPICIOUS_ACTIVITY_EVENT",
            "parameters": {"USER_EMAIL": "homer.simpson@example.io"},
        },
    ),
]


class GSuiteDeviceSuspiciousActivity(PantherRule):
    RuleID = "GSuite.DeviceSuspiciousActivity-prototype"
    DisplayName = "GSuite Device Suspicious Activity"
    LogTypes = [PantherLogType.GSuite_ActivityEvent]
    Tags = ["GSuite"]
    Severity = PantherSeverity.Low
    Description = "GSuite reported a suspicious activity on a user's device.\n"
    Reference = "https://support.google.com/a/answer/7562460?hl=en&sjid=864417124752637253-EU"
    Runbook = "Validate that the suspicious activity was expected by the user.\n"
    SummaryAttributes = ["actor:email"]
    Tests = g_suite_device_suspicious_activity_tests

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "mobile":
            return False
        return bool(event.get("name") == "SUSPICIOUS_ACTIVITY_EVENT")

    def title(self, event):
        return f"User [{deep_get(event, 'actor', 'email', default='<UNKNOWN_USER>')}]'s device was compromised"



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get
from pypanther.helpers.panther_default import aws_cloudtrail_success, lookup_aws_account_name
from pypanther.log_types import PantherLogType

aws_cloud_trail_stopped_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="CloudTrail Was Stopped",
        ExpectedResult=True,
        Log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/tester",
                        "accountId": "123456789012",
                        "userName": "Tester",
                    },
                    "webIdFederationData": {},
                    "attributes": {
                        "mfaAuthenticated": "true",
                        "creationDate": "2019-01-01T00:00:00Z",
                    },
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "cloudtrail.amazonaws.com",
            "eventName": "StopLogging",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "console.amazonaws.com",
            "requestParameters": {
                "name": "arn:aws:cloudtrail:us-west-2:123456789012:trail/example-trail"
            },
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "readOnly": False,
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    PantherRuleTest(
        Name="CloudTrail Was Started",
        ExpectedResult=False,
        Log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "111:panther-snapshot-scheduler",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "attributes": {
                        "mfaAuthenticated": "false",
                        "creationDate": "2019-01-01T00:00:00Z",
                    },
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/tester",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "cloudtrail.amazonaws.com",
            "eventName": "StartLogging",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": {
                "encryptionContext": {
                    "aws:lambda:FunctionArn": "arn:aws:lambda:us-west-2:123456789012:function:test-function"
                }
            },
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "readOnly": True,
            "resources": [
                {
                    "ARN": "arn:aws:kms:us-west-2:123456789012:key/1",
                    "accountId": "123456789012",
                    "type": "AWS::KMS::Key",
                }
            ],
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    PantherRuleTest(
        Name="Error Stopping CloudTrail",
        ExpectedResult=False,
        Log={
            "eventVersion": "1.05",
            "errorCode": "InvalidTrailNameException",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/tester",
                        "accountId": "123456789012",
                        "userName": "Tester",
                    },
                    "webIdFederationData": {},
                    "attributes": {
                        "mfaAuthenticated": "true",
                        "creationDate": "2019-01-01T00:00:00Z",
                    },
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "cloudtrail.amazonaws.com",
            "eventName": "StopLogging",
            "awsRegion": "us-west-2",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "console.amazonaws.com",
            "requestParameters": {
                "name": "arn:aws:cloudtrail:us-west-2:123456789012:trail/example-trail"
            },
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "readOnly": False,
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
]


class AWSCloudTrailStopped(PantherRule):
    RuleID = "AWS.CloudTrail.Stopped-prototype"
    DisplayName = "CloudTrail Stopped"
    LogTypes = [PantherLogType.AWS_CloudTrail]
    Tags = ["AWS", "Security Control", "DemoThreatHunting", "Defense Evasion:Impair Defenses"]
    Reports = {"CIS": ["3.5"], "MITRE ATT&CK": ["TA0005:T1562"]}
    Severity = PantherSeverity.Medium
    Description = "A CloudTrail Trail was modified.\n"
    Runbook = "https://docs.runpanther.io/alert-runbooks/built-in-rules/aws-cloudtrail-modified"
    Reference = "https://docs.aws.amazon.com/awscloudtrail/latest/userguide/cloudtrail-delete-trails-console.html"
    SummaryAttributes = [
        "eventName",
        "userAgent",
        "sourceIpAddress",
        "recipientAccountId",
        "p_any_aws_arns",
    ]
    Tests = aws_cloud_trail_stopped_tests
    # API calls that are indicative of CloudTrail changes
    CLOUDTRAIL_STOP_DELETE = {"DeleteTrail", "StopLogging"}

    def rule(self, event):
        return (
            aws_cloudtrail_success(event) and event.get("eventName") in self.CLOUDTRAIL_STOP_DELETE
        )

    def dedup(self, event):
        # Merge on the CloudTrail ARN
        return deep_get(event, "requestParameters", "name", default="<UNKNOWN_NAME>")

    def title(self, event):
        return f"CloudTrail [{self.dedup(event)}] in account [{lookup_aws_account_name(event.get('recipientAccountId'))}] was stopped/deleted"

    def alert_context(self, event):
        return aws_rule_context(event)



from typing import List

from pypanther.base import PantherRule, PantherRuleTest, PantherSeverity
from pypanther.log_types import PantherLogType

cisco_umbrella_dns_blocked_tests: List[PantherRuleTest] = [
    PantherRuleTest(
        Name="Domain Blocked",
        ExpectedResult=True,
        Log={
            "action": "Blocked",
            "internalIp": "136.24.229.58",
            "externalIp": "136.24.229.58",
            "timestamp": "2020-05-21 19:20:25.000",
            "responseCode": "NOERROR",
            "domain": "malware.gvt2.com.",
        },
    ),
    PantherRuleTest(
        Name="Action Allowed",
        ExpectedResult=False,
        Log={
            "action": "Allowed",
            "internalIp": "136.24.229.58",
            "externalIp": "136.24.229.58",
            "timestamp": "2020-05-21 19:20:25.000",
            "responseCode": "NOERROR",
            "domain": "beacons3.gvt2.com.",
        },
    ),
]


class CiscoUmbrellaDNSBlocked(PantherRule):
    RuleID = "CiscoUmbrella.DNS.Blocked-prototype"
    DisplayName = "Cisco Umbrella Domain Blocked"
    DedupPeriodMinutes = 480
    LogTypes = [PantherLogType.CiscoUmbrella_DNS]
    Tags = ["DNS"]
    Severity = PantherSeverity.Low
    Description = "Monitor blocked domains"
    Runbook = "Inspect the blocked domain and lookup for malware"
    Reference = "https://support.umbrella.com/hc/en-us/articles/230563627-How-to-determine-if-a-domain-or-resource-is-being-blocked-using-Chrome-Net-Internals"
    SummaryAttributes = ["action", "internalIp", "externalIp", "domain", "responseCode"]
    Tests = cisco_umbrella_dns_blocked_tests

    def rule(self, event):
        return event.get("action") == "Blocked"

    def title(self, event):
        return "Access denied to domain " + event.get("domain", "<UNKNOWN_DOMAIN>")



from typing import List

import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import PantherDataModel, PantherDataModelMapping
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_base_helpers import gsuite_details_lookup as details_lookup
from pypanther.log_types import PantherLogType


def get_event_type(event):
    # currently, only tracking a few event types
    # Pattern match this event to the recon actions
    if deep_get(event, "id", "applicationName") == "admin":
        if bool(details_lookup("DELEGATED_ADMIN_SETTINGS", ["ASSIGN_ROLE"], event)):
            return event_type.ADMIN_ROLE_ASSIGNED
    if details_lookup("login", ["login_failure"], event):
        return event_type.FAILED_LOGIN
    if deep_get(event, "id", "applicationName") == "login":
        return event_type.SUCCESSFUL_LOGIN
    return None


class StandardGSuiteReports(PantherDataModel):
    DataModelID: str = "Standard.GSuite.Reports"
    DisplayName: str = "GSuite Reports"
    Enabled: bool = True
    LogTypes: List[str] = [PantherLogType.GSuite_Reports]
    Mappings: List[PantherDataModelMapping] = [
        PantherDataModelMapping(Name="actor_user", Path="$.actor.email"),
        PantherDataModelMapping(
            Name="assigned_admin_role",
            Path="$.events[*].parameters[?(@.name == 'ROLE_NAME')].value",
        ),
        PantherDataModelMapping(Name="event_type", Method=get_event_type),
        PantherDataModelMapping(Name="source_ip", Path="ipAddress"),
        PantherDataModelMapping(
            Name="user", Path="$.events[*].parameters[?(@.name == 'USER_EMAIL')].value"
        ),
    ]



from typing import List

from pypanther.base import PantherDataModel, PantherDataModelMapping
from pypanther.log_types import PantherLogType


class StandardSlackAuditLogs(PantherDataModel):
    DataModelID: str = "Standard.Slack.AuditLogs"
    DisplayName: str = "Slack Audit Logs"
    Enabled: bool = True
    LogTypes: List[str] = [PantherLogType.Slack_AuditLogs]
    Mappings: List[PantherDataModelMapping] = [
        PantherDataModelMapping(Name="actor_user", Path="$.actor.user.name"),
        PantherDataModelMapping(Name="user_agent", Path="$.context.ua"),
        PantherDataModelMapping(Name="source_ip", Path="$.context.ip_address"),
        PantherDataModelMapping(Name="user", Path="$.entity.user.name"),
    ]



from typing import List

import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import PantherDataModel, PantherDataModelMapping
from pypanther.log_types import PantherLogType


def get_event_type(event):
    # pylint: disable=too-many-return-statements
    # pylint: disable=too-many-branches
    # pylint: disable=too-complex
    if event.get("category_type") == "User":
        if event.get("action") == "Add":
            return event_type.USER_ACCOUNT_CREATED
        if event.get("action") == "Delete":
            return event_type.USER_ACCOUNT_DELETED
        if event.get("action") == "Update" and "to Admin" in event.get("operation_detail"):
            return event_type.ADMIN_ROLE_ASSIGNED
        if event.get("action") == "Update":
            return event_type.USER_ACCOUNT_MODIFIED

    if event.get("category_type") == "User Group":
        if event.get("action") == "Add":
            return event_type.USER_GROUP_CREATED
        if event.get("action") == "Update":
            return event_type.USER_GROUP_MODIFIED
        if event.get("action") == "Delete":
            return event_type.USER_GROUP_DELETED

    if event.get("category_type") == "Role":
        if event.get("action") == "Add":
            return event_type.USER_ROLE_CREATED
        if event.get("action") == "Update":
            return event_type.USER_ROLE_MODIFIED
        if event.get("action") == "Delete":
            return event_type.USER_ROLE_DELETED
    return None


class StandardZoomOperation(PantherDataModel):
    DataModelID: str = "Standard.Zoom.Operation"
    DisplayName: str = None
    Enabled: bool = True
    LogTypes: List[str] = [PantherLogType.Zoom_Operation]
    Mappings: List[PantherDataModelMapping] = [
        PantherDataModelMapping(Name="actor_user", Path="operator"),
        PantherDataModelMapping(Name="event_type", Method=get_event_type),
    ]
